'''
Created on 2016/3/14

:author: hubo
'''
import sys
from vlcp.config.config import config, Configurable

try:
    from urllib import unquote, quote
    unquote_to_bytes = unquote
    quote_from_bytes = quote
except:
    from urllib.parse import unquote_to_bytes, quote_from_bytes
from namedstruct.namedstruct import NamedStruct, EmbeddedStruct
from namedstruct import dump as namedstruct_dump
from vlcp.utils.dataobject import DataObject, ReferenceObject, WeakReferenceObject, DataObjectSet
from vlcp.utils.dataobject import dump as dataobject_dump
import base64
import json

class JSONBytes(object):
    def __init__(self, data):
        self.data = data

def encode_default(obj):
    if isinstance(obj, JSONBytes):
        return {'<vlcpjsonencode/urlencoded-bytes>': quote_from_bytes(obj.data)}
    elif isinstance(obj, bytes):
        return {'<vlcpjsonencode/urlencoded-bytes>': quote_from_bytes(obj)}
    elif isinstance(obj, NamedStruct):
        # Hacked in the internal getstate implementation...
        state = obj.__getstate__()
        if state[2] is not obj:
            return {'<vlcpjsonencode/namedstruct.NamedStruct>':{'type':state[1], 'data':base64.b64encode(state[0]), 'target':state[2]}}
        else:
            return {'<vlcpjsonencode/namedstruct.NamedStruct>':{'type':state[1], 'data':base64.b64encode(state[0])}}
    else:
        if hasattr(obj, 'jsonencode'):
            try:
                key = '<vlcpjsonencode/' + type(obj).__module__ + '.' + type(obj).__name__ + '>'
            except AttributeError:
                raise TypeError(repr(obj) + " is not JSON serializable")
            else:
                return {key : obj.jsonencode()}
        else:
            raise TypeError(repr(obj) + " is not JSON serializable")


def decode_object(obj):
    if len(obj) == 1:
        try:
            k = str(next(iter(obj.keys())))
        except Exception:
            return obj
        v = obj[k]
        if k.startswith('<vlcpjsonencode/') and k.endswith('>'):
            classname = k[16:-1]
            if classname == 'urlencoded-bytes':
                return unquote_to_bytes(v)
            elif classname == 'namedstruct.NamedStruct':
                if 'target' in v:
                    target = v['target']
                    s = EmbeddedStruct.__new__(EmbeddedStruct)
                    s.__setstate__((base64.b64decode(v['data']), v['type'], target))
                else:
                    s = NamedStruct.__new__(NamedStruct)
                    s.__setstate__((base64.b64decode(v['data']), v['type'], s))
                return s
            else:
                dotpos = classname.rfind('.')
                if dotpos == -1:
                    raise ValueError(repr(classname) + ' is not a valid type')
                package = classname[:dotpos]
                cname = classname[dotpos+1:]
                p = None
                cls = None
                try:
                    p = sys.modules[package]
                    cls = getattr(p, cname)
                except KeyError:
                    raise ValueError(repr(classname) + ' is forbidden because it is not loaded')
                except AttributeError:
                    raise ValueError(repr(classname) + ' is not defined')
                if hasattr(cls, 'jsondecode'):
                    return cls.jsondecode(v)
                else:
                    raise ValueError(repr(classname) + ' is not JSON serializable')
        else:
            return obj
    else:
        return obj

@config('jsonformat')
class JsonFormat(Configurable):
    '''
    This is an extended JSON formatter used by WebAPI module
    '''
    # Enable special format for namedstruct (https://pypi.python.org/pypi/nstruct)
    _default_namedstruct = True
    # Use human read format, so:
    # namedstruct structures are formatted with readable names;
    # very long bytes are viewed like \<10000 bytes...\> (Python 3 only)
    _default_humanread = True
    # In Python 3, Try to decode bytes to str with this encoding 
    _default_bytesdecode = 'ascii'
    # When bytes object is longer than this and humanread=True, show \<*length* bytes...\>
    # instead of the exact content
    _default_byteslimit = 256
    # Dump extra information for namedstruct structures
    _default_dumpextra = False
    # Dump type information for namedstruct structures
    _default_dumptypeinfo = 'flat'
    # Dump the content for data objects
    _default_dataobject = True
    # Dump attributes for data objects
    _default_dataattributes = True    
    def jsonencoder(self, obj):
        if isinstance(obj, NamedStruct) and self.namedstruct:
            return namedstruct_dump(obj, self.humanread, self.dumpextra, self.dumptypeinfo)
        elif isinstance(obj, (DataObject, DataObjectSet, ReferenceObject, WeakReferenceObject)) and self.dataobject:
            return dataobject_dump(obj, self.dataattributes)
        elif isinstance(obj, bytes):
            if self.humanread and len(obj) > self.byteslimit:
                return '<%d bytes...>' % (len(obj),)
            else:
                if self.bytesdecode:
                    try:
                        return obj.decode(self.bytesdecode)
                    except:
                        return repr(obj)
                else:
                    return repr(obj)
        else:
            try:
                return encode_default(obj)
            except Exception:
                return repr(obj)
    


#! /usr/bin/python
# --*-- utf-8 --*--

from vlcp.utils.dataobject import DataObject,DataObjectSet

class PhysicalNetwork(DataObject):
    _prefix = 'viperflow.physicalnetwork'
    _indices = ("id",)

class PhysicalNetworkMap(DataObject):
    _prefix = 'viperflow.physicalnetworkmap'
    _indices = ("id",)
    
    def __init__(self,prefix = None,deleted = False):
        super(PhysicalNetworkMap,self).__init__(
                prefix = prefix,deleted = deleted)
        self.logicnetworks = DataObjectSet()
        self.network_allocation = dict()
        self.ports = DataObjectSet()

class PhysicalNetworkSet(DataObject):
    _prefix = 'viperflow.physicalnetworkset'

    def __init__(self,prefix = None,deleted = None):
        super(PhysicalNetworkSet,self).__init__(
                prefix = prefix,deleted = deleted)

        self.set = DataObjectSet()

class PhysicalPort(DataObject):
    _prefix = 'viperflow.physicalport'
    _indices = ('vhost','systemid','bridge','name')

class PhysicalPortSet(DataObject):
    _prefix = 'viperflow.physicalportset'

    def __init__(self,prefix = None,deleted = False):
        super(PhysicalPortSet,self).__init__(prefix = prefix,
                deleted = deleted)

        self.set = DataObjectSet()

class LogicalNetwork(DataObject):
    _prefix = 'viperflow.logicnetwork'
    _indices = ("id",)

class LogicalNetworkMap(DataObject):
    _prefix = 'viperflow.logicnetworkmap'
    _indices = ("id",)
    
    def __init__(self,prefix = None,deleted = None):
        super(LogicalNetworkMap,self).__init__(
                prefix = prefix,deleted = deleted)
        self.ports = DataObjectSet()
        self.subnets = DataObjectSet()

class LogicalNetworkSet(DataObject):
    _prefix = 'viperflow.logicalnetworkset'

    def __init__(self,prefix = None,deleted = None):
        super(LogicalNetworkSet,self).__init__(
                prefix = prefix,deleted = deleted)
        self.set = DataObjectSet()

class LogicalPort(DataObject):
    _prefix = 'viperflow.logicalport'
    _indices = ("id",)
    _unique_keys = (('_mac_address_index', ('network', 'mac_address')),
                    ('_ip_address_index', ('network', 'ip_address')))

class LogicalPortSet(DataObject):
    _prefix = 'viperflow.logcialportset'
    
    def __init__(self,prefix = None,deleted = None):
        super(LogicalPortSet,self).__init__(prefix = prefix,
                deleted = deleted)
        self.set = DataObjectSet()

class SubNet(DataObject):
    _prefix = 'viperflow.subnet'
    _indices = ('id',)


class SubNetMap(DataObject):
    _prefix = 'viperflow.subnetmap'
    _indices = ("id",)

    def __init__(self, prefix=None, deleted=None):
        super(SubNetMap, self).__init__(
                prefix=prefix, deleted=deleted)
        self.allocated_ips = dict()


class SubNetSet(DataObject):
    _prefix = 'viperflow.subnetset'

    def __init__(self, prefix=None, deleted=None):
        super(SubNetSet, self).__init__(prefix=prefix, deleted=deleted)
        self.set = DataObjectSet()


class VRouter(DataObject):
    _prefix = 'viperflow.vrouter'
    _indices = ('id',)

    def __init__(self,prefix=None,deleted=None):
        super(VRouter,self).__init__(prefix=prefix,deleted=deleted)
        self.interfaces = DataObjectSet()
        self.routes = list()


class DVRouterForwardSet(DataObject):
    _prefix = 'viperflow.dvrouterforwardset'

    def __init__(self,prefix=None,deleted=None):
        super(DVRouterForwardSet,self).__init__(prefix=prefix,deleted=deleted)
        self.set = DataObjectSet()

class DVRouterForwardInfo(DataObject):
    _prefix = 'viperflow.dvrouterforwardinfo'
    _indices = ("from_pynet","to_pynet")

    def __init__(self,prefix=None,deleted=None):
        super(DVRouterForwardInfo,self).__init__(prefix=prefix,deleted=deleted)
        self.info = []


class DVRouterForwardInfoRef(DataObject):
    _prefix = 'viperflow.dvrouterforwardrefinfo'
    _indices = ("from_pynet","to_pynet")

    def __init__(self,prefix=None,deleted=None):
        super(DVRouterForwardInfoRef,self).__init__(prefix=prefix,deleted=deleted)
        self.info = []


class DVRouterExternalAddressInfo(DataObject):
    _prefix = 'viperflow.dvrouterexternaladdressinfo'

    def __init__(self, prefix=None,deleted=None):
        super(DVRouterExternalAddressInfo,self).__init__(prefix=prefix,deleted=deleted)
        self.info = []

class VRouterSet(DataObject):
    _prefix = 'viperflow.vrouterset'

    def __init__(self,prefix=None,deleted=None):
        super(VRouterSet,self).__init__(prefix=prefix,deleted=deleted)
        self.set = DataObjectSet()


class RouterPort(DataObject):
    _prefix = 'viperflow.routerport'
    _indices = ('id',)


class VXLANEndpointSet(DataObject):
    _prefix = 'viperflow.vxlanendpointset'
    _indices = ('id',)    
    def __init__(self,prefix = None,deleted = None):
        super(VXLANEndpointSet, self).__init__(
                prefix = prefix,deleted = deleted)
        self.endpointlist = []


LogicalNetwork._register_auto_remove('VXLANEndpointSet', lambda x: [VXLANEndpointSet.default_key(x.id)])


class LogicalPortVXLANInfo(DataObject):
    _prefix = 'viperflow.logicalportvxlaninfo'
    _indices = ('id',)
    def __init__(self, prefix=None, deleted=False):
        super(LogicalPortVXLANInfo, self).__init__(prefix=prefix, deleted=deleted)
        self.endpoints = []

LogicalPort._register_auto_remove('LogicalPortVXLANInfo', lambda x: [LogicalPortVXLANInfo.default_key(x.id)])




'''
Created on 2015/06/01

:author: hubo
'''
from __future__ import print_function, absolute_import, division 

try:
    from vlcp_event_cython.matchtree import *
except Exception:

    class MatchTree(object):
        '''
        A dictionary tree for fast event match
        '''
        def __init__(self, parent = None):
            '''
            Constructor
            '''
            self.index = {}
            self.matchers = []
            self.parent = parent
            if parent is not None:
                self.depth = parent.depth + 1
            else:
                self.depth = 0
        def subtree(self, matcher, create = False):
            '''
            Find a subtree from a matcher
            
            :param matcher: the matcher to locate the subtree. If None, return the root of the tree.
            
            :param create: if True, the subtree is created if not exists; otherwise return None if not exists
            '''
            if matcher is None:
                return self
            current = self
            for i in range(self.depth, len(matcher.indices)):
                ind = matcher.indices[i]
                if ind is None:
                    # match Any
                    if hasattr(current, 'any'):
                        current = current.any
                    else:
                        if create:
                            cany = MatchTree(current)
                            cany.parentIndex = None
                            current.any = cany
                            current = cany
                        else:
                            return None
                else:
                    current2 = current.index.get(ind)
                    if current2 is None:
                        if create:
                            cind = MatchTree(current)
                            cind.parentIndex = ind 
                            current.index[ind] = cind
                            current = cind
                        else:
                            return None
                    else:
                        current = current2
            return current        
        def insert(self, matcher, obj):
            '''
            Insert a new matcher
            
            :param matcher: an EventMatcher
            
            :param obj: object to return
            '''
            current = self.subtree(matcher, True)
            current.matchers.append((matcher,obj))
            return current
        def remove(self, matcher, obj):
            '''
            Remove the matcher
            
            :param matcher: an EventMatcher
            
            :param obj: the object to remove
            '''
            current = self.subtree(matcher, False)
            if current is None:
                return
            current.matchers = list(t for t in current.matchers if t[0] is not matcher or t[1] is not obj)
            while not current.matchers and not hasattr(current,'any') \
                    and not current.index and current.parent is not None:
                # remove self from parents
                ind = current.parentIndex
                if ind is None:
                    del current.parent.any
                else:
                    del current.parent.index[ind]
                p = current.parent
                current.parent = None
                current = p
        def matchesWithMatchers(self, event):
            '''
            Return all matches for this event. The first matcher is also returned for each matched object.
            
            :param event: an input event
            '''
            ret = []
            self._matches(event, set(), ret)
            return tuple(ret)
        def matches(self, event):
            '''
            Return all matches for this event. The first matcher is also returned for each matched object.
            
            :param event: an input event
            '''
            ret = []
            self._matches(event, set(), ret)
            return tuple(r[0] for r in ret)
        def _matches(self, event, duptest, retlist):
            # 1. matches(self.index[ind], event)
            # 2. matches(self.any, event)
            # 3. self.matches
            if self.depth < len(event.indices):
                ind = event.indices[self.depth]
                if ind in self.index:
                    self.index[ind]._matches(event, duptest, retlist)
                if hasattr(self, 'any'):
                    self.any._matches(event, duptest, retlist)
            for m, o in self.matchers:
                if o not in duptest and m.judge(event):
                    duptest.add(o)
                    retlist.append((o, m))
        def matchfirst(self, event):
            '''
            Return first match for this event
            
            :param event: an input event
            '''
            # 1. matches(self.index[ind], event)
            # 2. matches(self.any, event)
            # 3. self.matches
            if self.depth < len(event.indices):
                ind = event.indices[self.depth]
                if ind in self.index:
                    m = self.index[ind].matchfirst(event)
                    if m is not None:
                        return m
                if hasattr(self, 'any'):
                    m = self.any.matchfirst(event)
                    if m is not None:
                        return m
            for m, o in self.matchers:
                if m is None or m.judge(event):
                    return o

    class EventTree(object):
        '''
        Store events; match matchers
        '''
        def __init__(self, parent = None, branch = 5):
            '''
            Constructor
            '''
            self.events = []
            self.subevents = []
            self.parent = parent
            if parent is not None:
                self.depth = parent.depth + 1
            else:
                self.depth = 0
            self.branch = 5
        def subtree(self, event, create = False):
            '''
            Find a subtree from an event
            '''
            current = self
            for i in range(self.depth, len(event.indices)):
                if not hasattr(current, 'index'):
                    return current
                ind = event.indices[i]
                if create:
                    current = current.index.setdefault(ind, EventTree(current, self.branch))
                    current.parentIndex = ind
                else:
                    current = current.index.get(ind)
                    if current is None:
                        return None
            return current
        def insert(self, event):
            current = self.subtree(event, True)
            if current.depth == len(event.indices):
                current.events.append(event)
            else:
                current.subevents.append(event)
                if len(current.subevents) > self.branch:
                    current.index = {}
                    for e in current.subevents:
                        current.insert(e)
                    del current.subevents[:]
        def _returnAll(self, ret):
            ret.extend(self.events)
            if hasattr(self, 'index'):
                for st in self.index.values():
                    st._returnAll(ret)
            else:
                ret.extend(self.subevents)
        def _removeFromParent(self):
            current = self
            while current.parent is not None:
                del current.parent.index[current.parentIndex]
                p = current.parent
                current.parent = None
                current = p
                if current.index or current.events:
                    break
            if hasattr(current, 'index') and not current.index:
                del current.index
        def _findAndRemove(self, matcher, ret):
            current = self
            while current.depth < len(matcher.indices):
                ind = matcher.indices[current.depth]
                if not hasattr(current, 'index'):
                    newsub = []
                    for e in current.subevents:
                        if matcher.isMatch(e, current.depth):
                            ret.append(e)
                        else:
                            newsub.append(e)
                    current.subevents = newsub
                    if not current.subevents and not current.events:
                        current._removeFromParent()
                    return
                elif ind is None:
                    for st in current.index.values():
                        st._findAndRemove(matcher, ret)
                    return
                else:
                    if ind not in current.index:
                        return
                    current = current.index[ind]
            current._returnAll(ret)
            current._removeFromParent()
        def findAndRemove(self, matcher):
            ret = []
            self._findAndRemove(matcher, ret)
            return tuple(ret)
        def remove(self, event):
            current = self.subtree(event)
            if current.depth == len(event.indices):
                current.events.remove(event)
            else:
                current.subevents.remove(event)

                



'''
Created on 2015/12/2

:author: hubo
'''
from vlcp.config import defaultconfig
from vlcp.server.module import Module, api, findModule
from vlcp.event.core import TimerEvent
from vlcp.event.runnable import RoutineContainer
from time import time
import os.path

@defaultconfig
class Manager(Module):
    '''
    Manage module loading/unloading. Optionally reload a module when modified.
    '''
    # Check files change with this interval.
    _default_checkinterval = 5
    # Automatically check the loaded module files, reload them if they are changed on the disk.
    # Notice that only the file contains the Module class is reloaded, other files will not be
    # reloaded automatically. You should reload them manually with *reloadmodules* API if necessary.
    _default_autoreload = False
    service = False
    def __init__(self, server):
        '''
        Constructor
        '''
        Module.__init__(self, server)
        self.apiroutine = RoutineContainer(self.scheduler)
        self.apiroutine.main = self._autoreload
        self._lastcheck = time()
        if self.autoreload:
            self.routines.append(self.apiroutine)
        self.createAPI(api(self.enableAutoReload),
                       api(self.activeModules),
                       api(self.reloadmodules, self.apiroutine),
                       api(self.loadmodule, self.apiroutine),
                       api(self.unloadmodule, self.apiroutine))
    def activeModules(self):
        "Return current loaded modules"
        return dict((k, v.getFullPath()) for k,v in self.server.moduleloader.activeModules.items())
    def _autoreload(self):
        th = self.scheduler.setTimer(self.checkinterval, self.checkinterval)
        try:
            tm = TimerEvent.createMatcher(th)
            while True:
                yield (tm,)
                t = time()
                reloads = []
                loaded = self.activeModules().values()
                self._logger.debug('Checking loaded modules: %r', loaded)
                for k in loaded:
                    p, _ = findModule(k, False)
                    if not p or not hasattr(p, '__file__') or not p.__file__:
                        continue
                    if p.__file__.endswith('.pyc'):
                        source = p.__file__[:-1]
                    else:
                        source = p.__file__
                    try:
                        mtime = os.path.getmtime(source)
                        if mtime <= t and mtime > self._lastcheck:
                            reloads.append(k)
                    except:
                        pass
                if reloads:
                    self._logger.warning('Auto reload following modules: %r', reloads)
                    try:
                        for m in self.reloadmodules(reloads):
                            yield m
                    except:
                        self._logger.warning('Exception occurs on auto reload', exc_info=True)
                self._lastcheck = t
        finally:
            self.scheduler.cancelTimer(th)
    def loadmodule(self, path):
        '''
        Load specified module
        
        :param path: module path (e.g. ``vlcp.service.connection.httpserver.HttpServer``)
        '''
        for m in self.apiroutine.delegateOther(self.server.moduleloader.loadByPath(path), self.server.moduleloader, ()):
            yield m
        self.apiroutine.retvalue = None
    def reloadmodules(self, pathlist):
        '''
        Reload specified modules.
        
        :param pathlist: list of module path
        '''
        for m in self.apiroutine.delegateOther(self.server.moduleloader.reloadModules(pathlist),
                                               self.server.moduleloader, ()):
            yield m
        self.apiroutine.retvalue = None
    def unloadmodule(self, path):
        '''
        Unload specified module
        
        :param path: module path (e.g. ``vlcp.service.connection.httpserver.HttpServer``)
        '''
        for m in self.apiroutine.delegateOther(self.server.moduleloader.unloadByPath(path),
                                               self.server.moduleloader, ()):
            yield m
        self.apiroutine.retvalue = None
    def enableAutoReload(self, enabled = True):
        '''
        Enable or disable auto reload.
        
        :param enabled: enable if True, disable if False
        '''
        enabled_now = self.apiroutine in self.routines
        if enabled != enabled_now:
            if enabled:
                self.apiroutine.start()
                self.routines.append(self.apiroutine)
            else:
                self.apiroutine.terminate()
                self.routines.remove(self.apiroutine)
        return None



#!/usr/bin/env python
'''
Created on 2015/11/17

:author: hubo
'''
try:
    import ez_setup
    ez_setup.use_setuptools()
except:
    pass
from setuptools import setup, find_packages

VERSION = '1.3.10'

setup(name='vlcp',
      version=VERSION,
      description='Full stack framework for SDN Controller, support Openflow 1.0, Openflow 1.3, and Nicira extensions. Also a powerful coroutine-based web server.',
      author='Hu Bo',
      author_email='hubo1016@126.com',
      license="http://www.apache.org/licenses/LICENSE-2.0",
      url='http://github.com/hubo1016/vlcp',
      keywords=['SDN', 'VLCP', 'Openflow'],
      test_suite = 'tests',
      use_2to3=False,
      install_requires = ["nstruct>=1.1.1"],
      packages=find_packages(exclude=("tests","tests.*","misc","misc.*")),
      entry_points={'console_scripts': ['vlcp-start = vlcp.start:default_start']})



'''
Created on 2016/3/21

:author: hubo
'''

from vlcp.server.module import Module, callAPI, depend, ModuleNotification,\
    ModuleLoadStateChanged, api, proxy
from vlcp.config import defaultconfig
import vlcp.service.connection.redisdb as redisdb
from vlcp.event.event import withIndices, Event
from vlcp.event.runnable import RoutineContainer
from vlcp.event.connection import ConnectionResetException
from vlcp.event.core import syscall_removequeue, QuitException
import json
from time import time
from zlib import compress, decompress, error as zlib_error
import functools
import uuid
import logging

@withIndices('notifier', 'transactid', 'keys', 'reason', 'fromself')
class UpdateNotification(Event):
    UPDATED = 'updated'
    RESTORED = 'restored'

@withIndices('notifier', 'stage')
class ModifyListen(Event):
    SUBSCRIBE = 'subscribe'
    LISTEN = 'listen'

def _delegate(func):
    @functools.wraps(func)
    def f(self, *args, **kwargs):
        for m in self.delegate(func(self, *args, **kwargs)):
            yield m
    return f

def _bytes(s):
    if isinstance(s, bytes):
        return s
    else:
        return s.encode('utf-8')

def _str(s):
    if isinstance(s, str):
        return s
    else:
        return s.decode('utf-8')

class _Notifier(RoutineContainer):
    _logger = logging.getLogger(__name__ + '.Notifier')
    def __init__(self, vhostbind, prefix, scheduler=None, singlecastlimit = 256, deflate = False):
        RoutineContainer.__init__(self, scheduler=scheduler, daemon=False)
        self.vhostbind = vhostbind
        self.prefix = _bytes(prefix)
        self._matchers = {}
        self._publishkey = uuid.uuid1().hex
        self._publishno = 1
        self._publish_wait = set()
        self._matchadd_wait = set()
        self._matchremove_wait = set()
        self._singlecastlimit = singlecastlimit
        self._deflate = deflate
    def main(self):
        try:
            timestamp = '%012x' % (int(time() * 1000),) + '-'
            transactno = 1
            for m in callAPI(self, 'redisdb', 'getclient', {'vhost':self.vhostbind}):
                yield m
            client, encoder, decoder = self.retvalue
            try:
                for m in client.subscribe(self, self.prefix):
                    yield m
            except Exception:
                _no_connection_start = True
            else:
                _no_connection_start = False
            self._matchers[b''] = self.retvalue[0]
            if self._deflate:
                oldencoder = encoder
                olddecoder = decoder
                def encoder(x):
                    return compress(oldencoder(x))
                def decoder(x):
                    try:
                        return olddecoder(decompress(x))
                    except zlib_error:
                        return olddecoder(x)
            if not _no_connection_start:
                self.subroutine(self._modifier(client), True, "modifierroutine")
            listen_modify = ModifyListen.createMatcher(self, ModifyListen.LISTEN)
            connection_down = client.subscribe_state_matcher(self, False)
            connection_up = client.subscribe_state_matcher(self, True)
            module_loaded = ModuleLoadStateChanged.createMatcher(state = ModuleLoadStateChanged.LOADED,
                                                 _ismatch = lambda x: x._instance.getServiceName() == 'redisdb')
            matchers = tuple(self._matchers.values()) + (listen_modify, connection_down)
            last_transact = None
            while True:
                if not _no_connection_start:
                    yield matchers
                if not _no_connection_start and self.matcher is listen_modify:
                    matchers = tuple(self._matchers.values()) + (listen_modify, connection_down)
                elif _no_connection_start or self.matcher is connection_down:
                    # Connection is down, wait for restore
                    # The module may be reloaded
                    if _no_connection_start:
                        recreate_matchers = True
                    else:
                        recreate_matchers = False
                    last_transact = None
                    while True:
                        yield (connection_up, module_loaded)
                        if self.matcher is module_loaded:
                            self.terminate(self.modifierroutine)
                            for m in callAPI(self, 'redisdb', 'getclient', {'vhost':self.vhostbind}):
                                yield m
                            client, encoder, decoder = self.retvalue
                            if self._deflate:
                                oldencoder = encoder
                                olddecoder = decoder
                                def encoder(x):
                                    return compress(oldencoder(x))
                                def decoder(x):
                                    try:
                                        return olddecoder(decompress(x))
                                    except zlib_error:
                                        return olddecoder(x)
                            # Recreate listeners
                            connection_down = client.subscribe_state_matcher(self, False)
                            connection_up = client.subscribe_state_matcher(self, True)
                            try:
                                for m in client.subscribe(self, *tuple(self._matchers.keys())):
                                    yield m
                            except Exception:
                                recreate_matchers = True
                                continue
                            else:
                                self._matchers = dict(zip(self._matchers.keys(), self.retvalue))
                                self.subroutine(self._modifier(client), True, "modifierroutine")
                                matchers = tuple(self._matchers.values()) + (listen_modify, connection_down)
                                break
                        else:
                            if recreate_matchers:
                                try:
                                    for m in client.subscribe(self, *[self.prefix + k for k in self._matchers.keys()]):
                                        yield m
                                except Exception:
                                    recreate_matchers = True
                                    continue
                                else:
                                    self._matchers = dict(zip(self._matchers.keys(), self.retvalue))
                                    self.subroutine(self._modifier(client), True, "modifierroutine")
                                    matchers = tuple(self._matchers.values()) + (listen_modify, connection_down)
                                    break
                            else:
                                matchers = tuple(self._matchers.values()) + (listen_modify, connection_down)
                                break
                    if self._publish_wait:
                        self.subroutine(self.publish())
                    transactid = '%s%016x' % (timestamp, transactno)
                    transactno += 1
                    def send_restore_notify(transactid):
                        if self._matchadd_wait or self._matchremove_wait:
                            # Wait for next subscribe success
                            for m in self.waitWithTimeout(1, ModifyListen.createMatcher(self, ModifyListen.LISTEN)):
                                yield m
                        for m in self.waitForSend(
                                UpdateNotification(self, transactid, tuple(self._matchers.keys()), UpdateNotification.RESTORED, False, extrainfo = None)):
                            yield m
                    self.subroutine(send_restore_notify(transactid), False)
                else:
                    transact = decoder(self.event.message)
                    if transact['id'] == last_transact:
                        # Ignore duplicated updates
                        continue
                    last_transact = transact['id']
                    pubkey, sep, pubno = last_transact.partition('-')
                    fromself = (sep and pubkey == self._publishkey)
                    transactid = '%s%016x' % (timestamp, transactno)
                    transactno += 1
                    self.subroutine(self.waitForSend(
                                UpdateNotification(self, transactid, tuple(_bytes(k) for k in transact['keys']), UpdateNotification.UPDATED, fromself, extrainfo = transact.get('extrainfo'))
                                                                           ), False)
        finally:
            if hasattr(self ,'modifierroutine') and self.modifierroutine:
                self.terminate(self.modifierroutine)
    def _modifier(self, client):
        try:
            modify_matcher = ModifyListen.createMatcher(self, ModifyListen.SUBSCRIBE)
            while True:
                try:
                    while self._matchadd_wait or self._matchremove_wait:
                        if self._matchadd_wait:
                            # Subscribe new keys
                            current_add = set(self._matchadd_wait)
                            self._matchadd_wait.clear()
                            add_keys = list(current_add.difference(self._matchers.keys()))
                            try:
                                for m in client.subscribe(self, *[self.prefix + k for k in add_keys]):
                                    yield m
                            except:
                                # Return to matchadd
                                self._matchadd_wait.update(current_add.difference(self._matchremove_wait))
                                raise
                            else:
                                self._matchers.update(zip(add_keys, self.retvalue))
                                for m in self.waitForSend(ModifyListen(self, ModifyListen.LISTEN)):
                                    yield m
                        if self._matchremove_wait:
                            # Unsubscribe keys
                            current_remove = set(self._matchremove_wait)
                            self._matchremove_wait.clear()
                            del_keys = list(current_remove.intersection(self._matchers.keys()))
                            try:
                                for m in client.unsubscribe(self, *[self.prefix + k for k in del_keys]):
                                    yield m
                            except:
                                # Return to matchremove
                                self._matchremove_wait.update(current_remove.difference(self._matchadd_wait))
                                raise
                            else:
                                for k in del_keys:
                                    del self._matchers[k]
                                for m in self.waitForSend(ModifyListen(self, ModifyListen.LISTEN)):
                                    yield m
                    yield (modify_matcher,)
                except (IOError, ConnectionResetException):
                    # Wait for connection resume
                    connection_up = client.subscribe_state_matcher(self)
                    yield (connection_up,)
        finally:
            self.subroutine(self._clearup(client, list(self._matchers.keys())))
    def _clearup(self, client, keys):
        try:
            if not self.scheduler.quitting:
                for m in client.unsubscribe(self, *[self.prefix + k for k in keys]):
                    yield m
        except Exception:
            pass
    def add_listen(self, *keys):
        keys = [_bytes(k) for k in keys]
        self._matchremove_wait.difference_update(keys)
        self._matchadd_wait.update(keys)
        for m in self.waitForSend(ModifyListen(self, ModifyListen.SUBSCRIBE)):
            yield m
    def remove_listen(self, *keys):        
        keys = [_bytes(k) for k in keys]
        self._matchadd_wait.difference_update(keys)
        self._matchremove_wait.update(keys)
        for m in self.waitForSend(ModifyListen(self, ModifyListen.SUBSCRIBE)):
            yield m
    @_delegate
    def publish(self, keys = (), extrainfo = None):
        keys = [_bytes(k) for k in keys]
        if self._publish_wait:
            merged_keys = list(self._publish_wait.union(keys))
            self._publish_wait.clear()
        else:
            merged_keys = list(keys)
        if not merged_keys:
            return
        for m in callAPI(self, 'redisdb', 'getclient', {'vhost':self.vhostbind}):
            yield m
        client, encoder, _ = self.retvalue
        transactid = '%s-%016x' % (self._publishkey, self._publishno)
        self._publishno += 1
        msg = encoder({'id':transactid, 'keys':[_str(k) for k in merged_keys], 'extrainfo': extrainfo})
        try:
            if len(merged_keys) > self._singlecastlimit:
                for m in client.execute_command(self, 'PUBLISH', self.prefix, msg):
                    yield m
            else:
                for m in client.batch_execute(self, *((('MULTI',),) +
                                                    tuple(('PUBLISH', self.prefix + k, msg) for k in merged_keys) +
                                                    (('EXEC',),))):
                    yield m
        except (IOError, ConnectionResetException):
            self._logger.warning('Following keys are not published because exception occurred, delay to next publish: %r', merged_keys, exc_info = True)
            self._publish_wait.update(merged_keys)
    def notification_matcher(self, fromself = None):
        if fromself is None:
            return UpdateNotification.createMatcher(self)
        else:
            return UpdateNotification.createMatcher(notifier = self, fromself = fromself)

@defaultconfig
@depend(redisdb.RedisDB)
class RedisNotifier(Module):
    """
    Update notification with Redis Pub/Sub
    """
    # Bind to RedisDB vHost
    _default_vhostbind = ''
    # Use this prefix for subscribe channels
    _default_prefix = 'vlcp.updatenotifier.'
    # If a notification contains more than .singlecastlimit keys, do not replicate the notification
    # to all channels; only broadcast it once in the public channel.
    _default_singlecastlimit = 256
    # Use an extra deflate compression on notification, should not be necessary if you already have
    # module.redisdb.deflate=True
    _default_deflate = False
    def __init__(self, server):
        Module.__init__(self, server)
        self.createAPI(api(self.createnotifier))
    def load(self, container):
        self.scheduler.queue.addSubQueue(999, ModifyListen.createMatcher(), "redisnotifier_modifylisten")
        for m in Module.load(self, container):
            yield m
    def unload(self, container, force=False):
        for m in Module.unload(self, container, force=force):
            yield m
        for m in container.syscall_noreturn(syscall_removequeue(self.scheduler.queue, "redisnotifier_modifylisten")):
            yield m
    def createnotifier(self):
        "Create a new notifier object"
        n = _Notifier(self.vhostbind, self.prefix, self.scheduler, self.singlecastlimit, self.deflate)
        n.start()
        return n

UpdateNotifier = proxy('UpdateNotifier', RedisNotifier)



'''
Created on 2015/7/24

:author: hubo
'''
from vlcp.config import Configurable, manager, config
from vlcp.event import Scheduler, DefaultPolling, RoutineControlEvent, CBQueue
from vlcp.utils.connector import Resolver
from vlcp.event import PollEvent, ConnectionWriteEvent, ConnectionControlEvent, StreamDataEvent
from vlcp.event.core import TimerEvent, SystemControlEvent, SystemControlLowPriorityEvent
from vlcp.event.connection import ResolveRequestEvent, ResolveResponseEvent
from vlcp.event.future import FutureEvent
from .module import Module, ModuleLoader, ModuleAPICall, ModuleAPIReply, ModuleNotification, ModuleLoadStateChanged
import logging
import logging.config
from vlcp.event.lock import LockEvent
import os

@config('server')
class Server(Configurable):
    '''
    Create a server with all necessary parts
    '''
    # Startup modules list. Should be a tuple of "package.classname" like::
    # 
    #    ('vlcp.service.sdn.viperflow.ViperFlow',
    #    'vlcp.service.sdn.vrouterapi.VRouterApi')
    #
    # Startup modules automatically load their dependencies, so it is not necessary
    # (though not an error) to write them explicitly.
    # 
    # If server.startup is null or empty, server tries to load modules
    # in __main__. 
    _default_startup = ()
    # enable debugging log for scheduler
    _default_debugging = False
    # File-can-write event priority, usually means socket send() can be used
    _default_pollwritepriority = 700
    # ConnectionWrite events priority
    _default_connectionwritepriority = 600
    # File-can-read event priority, usually means data received from socket
    _default_pollreadpriority = 500 
    # error event priority, usually means the socket is in an error status
    _default_pollerrorpriority = 800
    # responses from resolver
    _default_resolverresppriority = 490
    # requests to resolver
    _default_resolverreqpriority = 650
    # shutdown/reset/restart commands for connections
    _default_connectioncontrolpriority = 450
    # asynchronously starts a routine 
    _default_routinecontrolpriority = 1000
    # streams (vlcp.event.stream.Stream) data
    _default_streamdatapriority = 640
    # timers
    _default_timerpriority = 900
    # a lock can be acquired
    _default_lockpriority = 990
    # future objects been set
    _default_futurepriority = 989
    # a module is loaded/unloaded
    _default_moduleloadeventpriority = 890
    # system high-priority events
    _default_sysctlpriority = 2000
    # system low-priority events
    _default_sysctllowpriority = 10
    # module API call
    _default_moduleapicallpriority = 630
    # module API response
    _default_moduleapireplypriority = 420
    # module notify
    _default_modulenotifypriority = 410
    # default connection write queue limit for all connections
    _default_totalwritelimit = 100000
    # connection write events limit per connection
    _default_writelimitperconnection = 5
    # preserve space for newly created connections in default connection write queue
    _default_preservefornew = 100
    # stream data limit for all streams
    _default_streamdatalimit = 100000
    # stream data limit per stream
    _default_datalimitperstream = 5
    # the default multi-threading resolver pool size
    _default_resolverpoolsize = 64
    # try to set open files limit (ulimit -n) to this number if it is smaller
    _default_ulimitn = 32768
    # Use logging.config.dictConfig with this dictionary to configure the logging system.
    # It is supported in Python 2.7+
    _default_logging = None
    # Use logging.config.fileConfig with this file to configure the logging system.
    _default_loggingconfig = None
    # Force polling the sockets even if there are still unfinished events after processing
    # this number of events. May be helpful for very high stress. Should be set together
    # with server.queuemaxsize or/and server.queuedefaultsize to prevent memory overflow.
    _default_processevents = None
    # Limit the default queue size, so more events will be blocked until some events are
    # processed. This further limit the processing speed for producers to keep the system
    # stable on very high stress, but may slightly reduce the performance.
    _default_queuedefaultsize = None
    # Limit the total size of the event queue, so more events will be blocked until some events are
    # processed. This further limit the processing speed for producers to keep the system
    # stable on very high stress, but may slightly reduce the performance.
    _default_queuemaxsize = None
    def __init__(self):
        '''
        Constructor
        '''
        if hasattr(self, 'logging') and self.logging is not None:
            if isinstance(self.logging, dict):
                logging_config = dict(self.logging)
            else:
                logging_config = self.logging.todict()
            logging_config.setdefault('disable_existing_loggers', False)
            logging.config.dictConfig(logging_config)
        elif hasattr(self, 'loggingconfig') and self.loggingconfig is not None:
            logging.config.fileConfig(self.loggingconfig, disable_existing_loggers=False)
        self.scheduler = Scheduler(DefaultPolling(), getattr(self, 'processevents', None), getattr(self, 'queuedefaultsize', None), getattr(self, 'queuemaxsize', None),
                                   defaultQueueClass=CBQueue.AutoClassQueue.initHelper('_classname0'), defaultQueuePriority = 400)
        if self.debugging:
            self.scheduler.debugging = True
            self.scheduler.logger.setLevel(logging.DEBUG)
        self.scheduler.queue.addSubQueue(self.pollwritepriority, PollEvent.createMatcher(category=PollEvent.WRITE_READY), 'write', None, None, CBQueue.AutoClassQueue.initHelper('fileno'))
        self.scheduler.queue.addSubQueue(self.pollreadpriority, PollEvent.createMatcher(category=PollEvent.READ_READY), 'read', None, None, CBQueue.AutoClassQueue.initHelper('fileno'))
        self.scheduler.queue.addSubQueue(self.pollerrorpriority, PollEvent.createMatcher(category=PollEvent.ERROR), 'error')
        self.scheduler.queue.addSubQueue(self.connectioncontrolpriority, ConnectionControlEvent.createMatcher(), 'control')
        self.scheduler.queue.addSubQueue(self.connectionwritepriority, ConnectionWriteEvent.createMatcher(), 'connectionwrite', self.totalwritelimit, self.totalwritelimit, CBQueue.AutoClassQueue.initHelper('connection', self.preservefornew, subqueuelimit = self.writelimitperconnection))
        self.scheduler.queue.addSubQueue(self.streamdatapriority, StreamDataEvent.createMatcher(), 'streamdata', self.streamdatalimit, self.streamdatalimit, CBQueue.AutoClassQueue.initHelper('stream', self.preservefornew, subqueuelimit = self.datalimitperstream))
        self.scheduler.queue.addSubQueue(self.routinecontrolpriority, RoutineControlEvent.createMatcher(), 'routine')
        self.scheduler.queue.addSubQueue(self.timerpriority, TimerEvent.createMatcher(), 'timer')
        self.scheduler.queue.addSubQueue(self.resolverresppriority, ResolveResponseEvent.createMatcher(), 'resolve')
        self.scheduler.queue.addSubQueue(self.resolverreqpriority, ResolveRequestEvent.createMatcher(), 'resolvereq', 16)
        self.scheduler.queue.addSubQueue(self.sysctlpriority, SystemControlEvent.createMatcher(), 'sysctl')
        self.scheduler.queue.addSubQueue(self.sysctllowpriority, SystemControlLowPriorityEvent.createMatcher(), 'sysctllow')
        self.scheduler.queue.addSubQueue(self.moduleapicallpriority, ModuleAPICall.createMatcher(), 'moduleapi', None, None, CBQueue.AutoClassQueue.initHelper('target', 2, subqueuelimit = 5))
        self.scheduler.queue.addSubQueue(self.moduleapireplypriority, ModuleAPIReply.createMatcher(), 'moduleapireply')
        self.scheduler.queue.addSubQueue(self.modulenotifypriority, ModuleNotification.createMatcher(), 'modulenotify', None, None, CBQueue.AutoClassQueue.initHelper('target', subqueuelimit=5))
        self.scheduler.queue.addSubQueue(self.moduleloadeventpriority, ModuleLoadStateChanged.createMatcher(), 'moduleload')
        self.scheduler.queue.addSubQueue(self.lockpriority, LockEvent.createMatcher(), 'lock', None, None, CBQueue.AutoClassQueue.initHelper('key', subqueuelimit=1))
        self.scheduler.queue.addSubQueue(self.futurepriority, FutureEvent.createMatcher(), 'future')
        self.resolver = Resolver(self.scheduler, self.resolverpoolsize)
        self.moduleloader = ModuleLoader(self)
    def serve(self):
        """
        Start the server
        """
        if self.ulimitn is not None:
            try:
                import resource
                curr_ulimit = resource.getrlimit(resource.RLIMIT_NOFILE)
                if curr_ulimit[0] >= self.ulimitn:
                    # We do not decrease ulimit
                    pass
                elif curr_ulimit[1] >= self.ulimitn:
                    # Only increase soft limit, keep the hard limit unchanged
                    resource.setrlimit(resource.RLIMIT_NOFILE, (self.ulimitn, curr_ulimit[1]))
                else:
                    try:
                        resource.setrlimit(resource.RLIMIT_NOFILE, (self.ulimitn, self.ulimitn))
                    except:
                        # Maybe we do not have permission to change hard limit, instead we increase soft limit to the hard limit
                        resource.setrlimit(resource.RLIMIT_NOFILE, (curr_ulimit[1], curr_ulimit[1]))
            except:
                pass
        # If logging is not configured, configure it to the default (console)
        logging.basicConfig()
        self.resolver.start()
        self.moduleloader.start()
        for path in self.startup:
            self.moduleloader.subroutine(self.moduleloader.loadByPath(path))
        self.scheduler.main()

def main(configpath = None, startup = None, daemon = False, pidfile = None, fork = None):
    """
    The most simple way to start the VLCP framework
    
    :param configpath: path of a configuration file to be loaded
    
    :param startup: startup modules list. If None, `server.startup` in the configuration files
                    is used; if `server.startup` is not configured, any module defined or imported
                    into __main__ is loaded.
    
    :param daemon: if True, use python-daemon to fork and start at background. `python-daemon` must be
                   installed::
                    
                       pip install python-daemon
    
    :param pidfile: if daemon=True, this file is used for the pidfile.
    
    :param fork: use extra fork to start multiple instances
    """
    if configpath is not None:
        manager.loadfrom(configpath)
    if startup is not None:
        manager['server.startup'] = startup
    if not manager.get('server.startup'):
        # No startup modules, try to load from __main__
        startup = []
        import __main__
        for k in dir(__main__):
            m = getattr(__main__, k)
            if isinstance(m, type) and issubclass(m, Module) and m is not Module:
                startup.append('__main__.' + k)
        manager['server.startup'] = startup
    if fork is not None and fork > 1:
        if not hasattr(os, 'fork'):
            raise ValueError('Fork is not supported in this operating system.')
    def start_process():
        s = Server()
        s.serve()
    def main_process():
        if fork is not None and fork > 1:
            import multiprocessing
            from time import sleep
            sub_procs = []
            for i in range(0, fork):
                p = multiprocessing.Process(target = start_process)
                sub_procs.append(p)
            for i in range(0, fork):
                sub_procs[i].start()
            try:
                import signal
                def except_return(sig, frame):
                    raise SystemExit
                signal.signal(signal.SIGTERM, except_return)
                signal.signal(signal.SIGINT, except_return)
                if hasattr(signal, 'SIGHUP'):
                    signal.signal(signal.SIGHUP, except_return)
                while True:
                    sleep(2)
                    for i in range(0, fork):
                        if sub_procs[i].is_alive():
                            break
                    else:
                        break
            finally:
                for i in range(0, fork):
                    if sub_procs[i].is_alive():
                        sub_procs[i].terminate()
                for i in range(0, fork):
                    sub_procs[i].join()
        else:
            start_process()
    if daemon:
        import daemon
        if not pidfile:
            pidfile = manager.get('daemon.pidfile')
        uid = manager.get('daemon.uid')
        gid = manager.get('daemon.gid')
        if gid is None:
            group = manager.get('daemon.group')
            if group is not None:
                import grp
                gid = grp.getgrnam(group)[2]
        if uid is None:
            import pwd
            user = manager.get('daemon.user')
            if user is not None:
                user_pw = pwd.getpwnam(user)
                uid = user_pw.pw_uid
                if gid is None:
                    gid = user_pw.pw_gid
        if uid is not None and gid is None:
            import pwd
            gid = pwd.getpwuid(uid).pw_gid
        if pidfile:
            import fcntl
            class PidLocker(object):
                def __init__(self, path):
                    self.filepath = path
                    self.fd = None
                def __enter__(self):
                    # Create pid file
                    self.fd = os.open(pidfile, os.O_WRONLY | os.O_TRUNC | os.O_CREAT, 0o644)
                    fcntl.lockf(self.fd, fcntl.LOCK_EX|fcntl.LOCK_NB)
                    os.write(self.fd, str(os.getpid()).encode('ascii'))
                    os.fsync(self.fd)
                def __exit__(self, typ, val, tb):
                    if self.fd:
                        try:
                            fcntl.lockf(self.fd, fcntl.LOCK_UN)
                        except:
                            pass
                        os.close(self.fd)
                        self.fd = None
            locker = PidLocker(pidfile)
        else:
            locker = None
        import sys
        # Module loading is related to current path, add it to sys.path
        cwd = os.getcwd()
        if cwd not in sys.path:
            sys.path.append(cwd)
        # Fix path issues on already-loaded modules
        for m in sys.modules.values():
            if getattr(m, '__path__', None):
                m.__path__ = [os.path.abspath(p) for p in m.__path__]
            # __file__ is used for module-relative resource locate
            if getattr(m, '__file__', None):
                m.__file__ = os.path.abspath(m.__file__)
        configs = {'gid':gid,'uid':uid,'pidfile':locker}
        config_filters = ['chroot_directory', 'working_directory', 'umask', 'detach_process',
                          'prevent_core']
        if hasattr(manager, 'daemon'):
            configs.update((k,v) for k,v in manager.daemon.config_value_items() if k in config_filters)
        if not hasattr(os, 'initgroups'):
            configs['initgroups'] = False
        with daemon.DaemonContext(**configs):
            main_process()
    else:
        main_process()



'''
/* Copyright (c) 2008, 2011, 2012, 2013, 2014 The Board of Trustees of The Leland Stanford
 * Junior University
 *
 * We are making the OpenFlow specification and associated documentation
 * (Software) available for public use and benefit with the expectation
 * that others will use, modify and enhance the Software and contribute
 * those enhancements back to the community. However, since we would
 * like to make the Software available for broadest use, with as few
 * restrictions as possible permission is hereby granted, free of
 * charge, to any person obtaining a copy of this Software to deal in
 * the Software under the copyrights without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * The name and trademarks of copyright holder(s) may NOT be used in
 * advertising or publicity pertaining to the Software or any
 * derivatives without specific, written prior permission.
 */

/*
 * Copyright (c) 2008-2014 Nicira, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Created on 2015/7/13

:author: hubo
'''
from namedstruct import *
from namedstruct.namedstruct import NamedStruct, StructDefWarning
from vlcp.utils.ethernet import *
import warnings as _warnings

with _warnings.catch_warnings():
    _warnings.filterwarnings('ignore', '^padding', StructDefWarning)
    ofp_version =  enum('ofp_version',
                        globals(),
                        uint8,
                        OFP10_VERSION = 0x01,
                        OFP11_VERSION = 0x02,
                        OFP12_VERSION = 0x03,
                        OFP13_VERSION = 0x04,
                        OFP14_VERSION = 0x05,
                        OFP15_VERSION = 0x06)
    
    ofp_version_bitwise = enum('ofp_version_bitwise', None, uint32, True,
                               **dict((k, 1<<v) for k,v in ofp_version.getDict().items()))
    
    OF_VENDOR_ID = 0
    NX_VENDOR_ID = 0x00002320
    ONF_VENDOR_ID = 0x4f4e4600
    
    OFP_MAX_TABLE_NAME_LEN = 32
    OFP_MAX_PORT_NAME_LEN = 16
    
    OFP_PORT = 6653
    OFP_TCP_PORT = 6653
    OFP_SSL_PORT = 6653
    
    OFP_ETH_ALEN = 6
    
    ofp_type = enum('ofp_type', globals(), uint8,
        OFPT_HELLO              = 0,  #/* Symmetric message */
        OFPT_ERROR              = 1,  #/* Symmetric message */
        OFPT_ECHO_REQUEST       = 2,  #/* Symmetric message */
        OFPT_ECHO_REPLY         = 3,  #/* Symmetric message */
    )
    
    OFP_DEFAULT_MISS_SEND_LEN = 128
    
    OFP_DL_TYPE_ETH2_CUTOFF = 0x0600
    
    OFP_DL_TYPE_NOT_ETH_TYPE = 0x05ff
    
    OFP_FLOW_PERMANENT = 0
    
    OFP_DEFAULT_PRIORITY = 0x8000
    
    ofp_header = nstruct((ofp_version, 'version'),
                         (ofp_type, 'type'),
                         (uint16, 'length'),
                         (uint32, 'xid'),
                         name = 'ofp_header')
    
    ofp_msg = nstruct((ofp_header, 'header'),
                      name = 'ofp_msg',
                      padding = 1,
                      size = sizefromlen(65536, 'header', 'length'),
                      prepack = packrealsize('header', 'length'))
    
    ofp_msg_mutable = nstruct(
        name = 'ofp_msg_mutable',
        base = ofp_msg,
        criteria = lambda x: x.header.type > OFPT_ECHO_REPLY,
        classifier = lambda x: x.header.version
    )
    
    NamedStruct._registerPickleType('protocol.openflow.defs.common.ofp_msg', ofp_msg)
    
    ofp_error_type = enum('ofp_error_type', globals(), uint16,
        OFPET_HELLO_FAILED         = 0,  #/* Hello protocol failed. */
        OFPET_BAD_REQUEST          = 1,  #/* Request was not understood. */
        OFPET_BAD_ACTION           = 2,  #/* Error in action description. */
        NXET_VENDOR = 0xb0c2,
        OFPET_EXPERIMENTER = 0xffff
    )
    
    ofp_error_msg_base = nstruct((ofp_error_type, 'type'),
                            name = 'ofp_error_msg_base',
                            base = ofp_msg,
                            criteria = lambda x: x.header.type == OFPT_ERROR,
                            init = packvalue(OFPT_ERROR, 'header', 'type'))
    
    ofp_error_msg = nstruct((uint16, 'code'),
                            (raw, 'data'),
                            name = 'ofp_error_msg',
                            base = ofp_error_msg_base,
                            criteria = lambda x: x.type != OFPET_EXPERIMENTER or x.header.length < 14)
    
    ofp_error_experimenter_msg = nstruct(
        (uint16, 'exp_type'),
        (uint32, 'experimenter'),  #  /* Experimenter ID which takes the same form as in struct ofp_experimenter_header. */
    #    (raw, 'data'),          # /* Variable-length data.  Interpreted based on the type and code.  No padding. */
        name = 'ofp_error_experimenter_msg',
        base = ofp_error_msg_base,
        criteria = lambda x: x.header.length >= 14
    )
    
    '''
    /* ofp_error msg 'code' values for NXET_VENDOR. */
    '''
    nx_vendor_code = enum('nx_vendor_code', globals(), uint16,
        NXVC_VENDOR_ERROR = 0,         #  /* 'data' contains struct nx_vendor_error. */
    )
    
    '''
    /* ofp_error_msg 'code' values for OFPET_HELLO_FAILED.  'data' contains an
     * ASCII text string that may give failure details. */
    '''
    ofp_hello_failed_code = enum('ofp_hello_failed_code', globals(),
        OFPHFC_INCOMPATIBLE = 0, #   /* No compatible version. */
        OFPHFC_EPERM        = 1, #   /* Permissions error. */
    )
    
    def ofp_error_typedef(subtype, codeenum, version = None, extendtype = None):
        if version is None:
            return nstruct(name = 'ofp_error_msg',
                           base = ofp_error_msg,
                           criteria = lambda x: x.type == subtype,
                           init = packvalue(subtype, 'type'),
                           extend = {'code' : codeenum}
                           )
        else:
            ext = {'code' : codeenum}
            if extendtype is not None:
                ext['type'] = extendtype
            return nstruct(name = 'ofp_error_msg',
                           base = ofp_error_msg,
                           criteria = lambda x: x.type == subtype and x.header.version == version,
                           init = packvalue(subtype, 'type'),
                           extend = ext
                           )
    
    '''
    /* ofp_error_msg 'code' values for OFPET_BAD_REQUEST.  'data' contains at least
     * the first 64 bytes of the failed request. */
    '''
    ofp_bad_request_code = enum('ofp_bad_request_code', globals(),
        OFPBRC_BAD_VERSION      = 0, # /* ofp_header.version not supported. */
        OFPBRC_BAD_TYPE         = 1, # /* ofp_header.type not supported. */
        OFPBRC_BAD_MULTIPART    = 2, # /* ofp_multipart_request.type not supported. */
        OFPBRC_BAD_EXPERIMENTER = 3, # /* Experimenter id not supported
    #                                   * (in ofp_experimenter_header or
    #                                   * ofp_multipart_request or
    #                                   * ofp_multipart_reply). */
        OFPBRC_BAD_EXP_TYPE     = 4, # /* Experimenter type not supported. */
        OFPBRC_EPERM            = 5, # /* Permissions error. */
        OFPBRC_BAD_LEN          = 6, # /* Wrong request length for type. */
        OFPBRC_BUFFER_EMPTY     = 7, # /* Specified buffer has already been used. */
        OFPBRC_BUFFER_UNKNOWN   = 8, # /* Specified buffer does not exist. */
        OFPBRC_BAD_TABLE_ID     = 9, # /* Specified table-id invalid or does not
    #                                   * exist. */
        OFPBRC_IS_SLAVE         = 10,# /* Denied because controller is slave. */
        OFPBRC_BAD_PORT         = 11,# /* Invalid port. */
        OFPBRC_BAD_PACKET       = 12,# /* Invalid packet in packet-out. */
        OFPBRC_MULTIPART_BUFFER_OVERFLOW    = 13, #/* ofp_multipart_request
    #                                     overflowed the assigned buffer. */
    )
    OFPBRC_BAD_STAT = OFPBRC_BAD_MULTIPART
    OFPBRC_BAD_VENDOR = OFPBRC_BAD_EXPERIMENTER
    OFPBRC_BAD_SUBTYPE = OFPBRC_BAD_EXP_TYPE
    
    
    '''
    /* ofp_error_msg 'code' values for OFPET_BAD_ACTION.  'data' contains at least
     * the first 64 bytes of the failed request. */
    '''
    ofp_bad_action_code = enum('ofp_bad_action_code', globals(),
        OFPBAC_BAD_TYPE           = 0, # /* Unknown or unsupported action type. */
        OFPBAC_BAD_LEN            = 1, # /* Length problem in actions. */
        OFPBAC_BAD_EXPERIMENTER   = 2, # /* Unknown experimenter id specified. */
        OFPBAC_BAD_EXP_TYPE       = 3, # /* Unknown action for experimenter id. */
        OFPBAC_BAD_OUT_PORT       = 4, # /* Problem validating output port. */
        OFPBAC_BAD_ARGUMENT       = 5, # /* Bad action argument. */
        OFPBAC_EPERM              = 6, # /* Permissions error. */
        OFPBAC_TOO_MANY           = 7, # /* Can't handle this many actions. */
        OFPBAC_BAD_QUEUE          = 8, # /* Problem validating output queue. */
        OFPBAC_BAD_OUT_GROUP      = 9, # /* Invalid group id in forward action. */
        OFPBAC_MATCH_INCONSISTENT = 10,# /* Action can't apply for this match,
    #                                       or Set-Field missing prerequisite. */
        OFPBAC_UNSUPPORTED_ORDER  = 11,# /* Action order is unsupported for the
    #                                 action list in an Apply-Actions instruction */
        OFPBAC_BAD_TAG            = 12,# /* Actions uses an unsupported
                                       #    tag/encap. */
        OFPBAC_BAD_SET_TYPE       = 13,# /* Unsupported type in SET_FIELD action. */
        OFPBAC_BAD_SET_LEN        = 14,# /* Length problem in SET_FIELD action. */
        OFPBAC_BAD_SET_ARGUMENT   = 15,# /* Bad argument in SET_FIELD action. */
    )
    
    ofp_error_types = {OFPET_HELLO_FAILED: ofp_error_typedef(OFPET_HELLO_FAILED, ofp_hello_failed_code),
                       OFPET_BAD_REQUEST: ofp_error_typedef(OFPET_BAD_REQUEST, ofp_bad_request_code),
                       OFPET_BAD_ACTION: ofp_error_typedef(OFPET_BAD_ACTION, ofp_bad_action_code),
                       NXET_VENDOR: ofp_error_typedef(NXET_VENDOR, nx_vendor_code)
                       }
    
    ofp_config_flags = enum('ofp_config_flags',
                            globals(),
                            uint16,
                            True,
                            OFPC_FRAG_NORMAL = 0,
                            OFPC_FRAG_DROP = 1,
                            OFPC_FRAG_REASM = 2,
                            OFPC_FRAG_NX_MATCH = 3,
                            OFPC_INVALID_TTL_TO_CONTROLLER = 1 << 2)
    
    OFPC_FRAG_MASK = 3,
    
    
    ofp_port_config = enum('ofp_port_config',
                           globals(),
                           uint32,
                           True,
                           OFPPC_PORT_DOWN = 1<<0,
                           OFPPC_NO_STP = 1<<1,
                           OFPPC_NO_RECV = 1<<2,
                           OFPPC_NO_RECV_STP = 1<<3,
                           OFPPC_NO_FLOOD = 1<<4,
                           OFPPC_NO_FWD = 1<<5,
                           OFPPC_NO_PACKET_IN = 1<<6)
    
    ofp_port_state = enum('ofp_port_state',
                          globals(),
                          uint32,
                          True,
                          OFPPS_LINK_DOWN = 1<<0)
    
    ofp_port_features = enum('ofp_port_features',
                             globals(),
                             uint32,
                             True,
                             OFPPF_10MB_HD = 1<<0,
                             OFPPF_10MB_FD = 1<<1,
                             OFPPF_100MB_HD = 1<<2,
                             OFPPF_100MB_FD = 1<<3,
                             OFPPF_1GB_HD = 1<<4,
                             OFPPF_1GB_FD = 1<<5,
                             OFPPF_10GB_FD = 1<<6)
    
    ofp_queue_properties = enum('ofp_queue_properties',
                                globals(),
                                uint16,
                                OFPQT_MIN_RATE = 1,
                                OFPQT_MAX_RATE = 2,
                                OFPQT_EXPERIMENTER = 0xffff)
    
    
    ofp_capabilities = enum('ofp_capabilities',
                            globals(),
                            uint32,
                            True,
                            OFPC_FLOW_STATS = 1<<0,
                            OFPC_TABLE_STATS = 1<<1,
                            OFPC_PORT_STATS = 1<<2,
                            OFPC_IP_REASM = 1<<5,
                            OFPC_QUEUE_STATS = 1<<6,
                            OFPC_ARP_MATCH_IP = 1<<7)
    
    ofp_packet_in_reason = enum('ofp_packet_in_reason',
                                globals(),
                                uint8,
                                OFPR_NO_MATCH = 0,
                                OFPR_ACTION = 1,
                                OFPR_INVALID_TTL = 2
                                )
    OFPR_N_REASONS = 3
    
    ofp_flow_mod_command = enum('ofp_flow_mod_command',
                                globals(),
                                uint16,
                                OFPFC_ADD = 0,
                                OFPFC_MODIFY = 1,
                                OFPFC_MODIFY_STRICT = 2,
                                OFPFC_DELETE = 3,
                                OFPFC_DELETE_STRICT = 4)
    
    ofp_flow_mod_flags = enum('ofp_flow_mod_flags',
                              globals(),
                              uint16,
                              True,
                              OFPFF_SEND_FLOW_REM = 1 << 0,
                              OFPFF_CHECK_OVERLAP = 1 << 1                          
                              )
    
    ofp_flow_removed_reason = enum('ofp_flow_removed_reason',
                                   globals(),
                                   uint8,
                                   OFPRR_IDLE_TIMEOUT = 0,
                                   OFPRR_HARD_TIMEOUT = 1,
                                   OFPRR_DELETE = 2,
                                   OFPRR_GROUP_DELETE = 3,
                                   OFPRR_METER_DELETE = 4,
                                   OFPRR_EVICTION = 5)
    
    ofp_port_reason = enum('ofp_port_reason',
                           globals(),
                           uint8,
                           OFPPR_ADD = 0,
                           OFPPR_DELETE = 1,
                           OFPPR_MODIFY = 2)
    
    ofp_stats_reply_flags = enum('ofp_stats_reply_flags',
                                 globals(),
                                 uint16,
                                 OFPSF_REPLY_MORE = 1<<0)
    
    
    ofp_aggregate_stats_reply = nstruct((uint64, 'packet_count'),
                                        (uint64, 'byte_count'),
                                        (uint32, 'flow_count'),
                                        (uint8[4],),
                                        name = 'ofp_aggregate_stats_reply')
    
    ofp_match_type = enum('ofp_match_type',
                          globals(),
                          uint16,
                          OFPMT_STANDARD = 0,
                          OFPMT_OXM = 1)
    
    ofp_group = enum('ofp_group',
                     globals(),
                     uint32,
                     OFPG_MAX = 0xffffff00,
                     OFPG_ALL = 0xfffffffc,
                     OFPG_ANY = 0xffffffff)
    
    ofp_group_capabilities = enum('ofp_group_capabilities',
                                  globals(),
                                  OFPGFC_SELECT_WEIGHT = 1 << 0,
                                  OFPGFC_SELECT_LIVENESS = 1 << 1,
                                  OFPGFC_CHAINING = 1 << 2,
                                  OFPGFC_CHAINING_CHECKS = 1 << 3)
    
    ofp_hello_elem_type = enum('ofp_hello_elem_type',
                               globals(),
                               uint16,
                               OFPHET_VERSIONBITMAP = 1) 
    
    ofp_hello_elem = nstruct((ofp_hello_elem_type, 'type'),
                             (uint16, 'length'),
                             name = 'ofp_hello_elem',
                             size = sizefromlen(32, 'length'),
                             prepack = packrealsize('length'),
                             padding = 8
                             )
    
    ofp_versionbitmap = uint32[0]
    
    ofp_versionbitmap.formatter = lambda x: ofp_version_bitwise.tostr(sum(x[i] << (i * 32) for i in range(0, len(x))))
    
    ofp_hello_elem_versionbitmap = nstruct(
                            (ofp_versionbitmap, 'bitmaps'),
                            name = 'ofp_hello_elem_versionbitmap',
                            base = ofp_hello_elem,
                            criteria = lambda x: x.type == OFPHET_VERSIONBITMAP,
                            init = packvalue(OFPHET_VERSIONBITMAP, 'type')
                                           )
    
    ofp_hello = nstruct(
                    (ofp_hello_elem[0], 'elements'),
                    name = 'ofp_hello',
                    base = ofp_msg,
                    criteria = lambda x: x.header.type == OFPT_HELLO,
                    init = packvalue(OFPT_HELLO, 'header', 'type')
                        )
    
    ofp_echo = nstruct(
                       (raw, 'data'),
                       name = 'ofp_echo',
                       base = ofp_msg,
                       criteria = lambda x: x.header.type == OFPT_ECHO_REQUEST or x.header.type == OFPT_ECHO_REPLY,
                       init = packvalue(OFPT_ECHO_REQUEST, 'header', 'type')
                       )
    
    
    
    ofp_table = enum('ofp_table',
                     globals(),
                     uint8,
                     OFPTT_ALL = 0xff)
    
    OFPTT_MAX = 0xfe
    
    
    ofp_table_config = enum('ofp_table_config',
                            globals(),
                            uint32,
                            True,
                            OFPTC_TABLE_MISS_CONTROLLER = 0 << 0, #/* Send to controller. */
                            OFPTC_TABLE_MISS_CONTINUE   = 1 << 0, #/* Go to next table, like OF1.0. */
                            OFPTC_TABLE_MISS_DROP       = 2 << 0, #/* Drop the packet. */
                            OFPTC_TABLE_MISS_MASK       = 3 << 0,
                        
                            #/* OpenFlow 1.4. */
                            OFPTC_EVICTION              = 1 << 2, #/* Allow table to evict flows. */
                            OFPTC_VACANCY_EVENTS        = 1 << 3) #/* Enable vacancy events. */



'''
Created on 2016/10/20

:author: hubo
'''
from __future__ import print_function

import vlcp.utils.jsonencoder
from contextlib import closing

def decode_object(obj):
    return obj

vlcp.utils.jsonencoder.decode_object = decode_object

from vlcp.scripts.script import ScriptModule
from vlcp.server.module import findModule, callAPI

class MigrateDB(ScriptModule):
    '''
    Migrate data from one database to another
    
    migratedb.py -f <configfile> [--clean] src_module[:src_vhost] dst_module[:dst_vhost]
    
    src_module, dst_module is one of: redisdb, zookeeperdb, defaultdb
    '''
    _default_knownmodules = {'redisdb': 'vlcp.service.connection.redisdb.RedisDB',
                             'zookeeperdb': 'vlcp.service.connection.zookeeperdb.ZooKeeperDB',
                             'defaultdb': 'vlcp.service.kvdb.storage.KVStorage'}
    options = (('clean', None, False),)
    def run(self, src, dst, clean = None):
        if clean is not None:
            clean = True
        else:
            clean = False
        src_module, sep, src_vhost = src.partition(':')
        dst_module, sep, dst_vhost = dst.partition(':')
        load_modules = [self.knownmodules.get(m, m) for m in set((src_module, dst_module))]
        with closing(self.apiroutine.executeAll([self.server.moduleloader.loadByPath(m) for m in load_modules], self.server.moduleloader,
                                            ())) as g:
            for m in g:
                yield m
        src_service = findModule(self.knownmodules.get(src_module, src_module))[1]._instance.getServiceName()
        dst_service = findModule(self.knownmodules.get(dst_module, dst_module))[1]._instance.getServiceName()
        if clean:
            with closing(self.apiroutine.executeAll([callAPI(self.apiroutine, src_service, 'listallkeys', {'vhost': src_vhost}),
                                                 callAPI(self.apiroutine, dst_service, 'listallkeys', {'vhost': dst_vhost})])) as g:
                for m in g:
                    yield m
            (src_keys,), (dst_keys,) = self.apiroutine.retvalue
        else:
            for m in callAPI(self.apiroutine, src_service, 'listallkeys', {'vhost': src_vhost}):
                yield m
            src_keys = self.apiroutine.retvalue
            dst_keys = []
        delete_keys = set(dst_keys)
        delete_keys.difference_update(src_keys)
        print('Migrating from %s (vhost = %r) to %s (vhost = %r)' % (src_service, src_vhost, dst_service, dst_vhost))
        print('Moving %d keys, deleting %d keys' % (len(src_keys), len(delete_keys)))
        print('Please check. Migrating starts in 5 seconds, press Ctrl+C to cancel...')
        for m in self.apiroutine.waitWithTimeout(5):
            yield m
        for i in range(0, len(src_keys), 100):
            move_keys = tuple(src_keys[i:i+100])
            try:
                for m in callAPI(self.apiroutine, src_service, 'mget', {'keys': move_keys,
                                                                        'vhost': src_vhost}):
                    yield m
                values = self.apiroutine.retvalue
            except ValueError:
                # There might be illegal keys, try them one by one
                def move_key(key):
                    try:
                        for m in callAPI(self.apiroutine, src_service, 'mget', {'keys': (key,),
                                                                                'vhost': src_vhost}):
                            yield m
                    except ValueError:
                        print('Key %r is not valid, it cannot be loaded. Ignore this key.' % (key,))
                    else:
                        for m in callAPI(self.apiroutine, dst_service, 'mset', {'kvpairs': ((key, self.apiroutine.retvalue[0]),),
                                                                                'vhost': dst_vhost}):
                            yield m
                with closing(self.apiroutine.executeAll([move_key(k) for k in move_keys], retnames = ())) as g:
                    for m in g:
                        yield m
            else:
                for m in callAPI(self.apiroutine, dst_service, 'mset', {'kvpairs': tuple(zip(move_keys, values)),
                                                                        'vhost': dst_vhost}):
                    yield m
            print('%d keys moved' % (i,))
        print('Deleting old keys...')
        if delete_keys:
            for k in delete_keys:
                for m in callAPI(self.apiroutine, dst_service, 'delete', {'key': k,
                                                                          'vhost': dst_vhost}):
                    yield m
        print('All jobs done.')

if __name__ == '__main__':
    MigrateDB.main()



'''
Created on 2015/8/12

:author: hubo
'''

from vlcp.protocol import Protocol
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices, ConnectionWriteEvent
import logging
import os
import json
import re

@withIndices('state', 'connection', 'connmark', 'createby')
class JsonRPCConnectionStateEvent(Event):
    """
    Connection state change
    """
    # Connection up
    CONNECTION_UP = 'up'
    # Connection down
    CONNECTION_DOWN = 'down'


@withIndices('method', 'connection', 'connmark', 'createby')
class JsonRPCRequestEvent(Event):
    """
    Request received from the connection
    """
    canignore = False
    def canignorenow(self):
        return not self.connection.connected or self.connection.connmark != self.connmark

@withIndices('connection', 'connmark', 'id', 'iserror', 'createby')
class JsonRPCResponseEvent(Event):
    """
    Response received from the connection
    """
    pass

@withIndices('method', 'connection', 'connmark', 'createby')
class JsonRPCNotificationEvent(Event):
    """
    Notification received from the connection
    """
    pass

class JsonFormatException(Exception):
    pass

class JsonRPCProtocolException(Exception):
    pass

class JsonRPCErrorResultException(Exception):
    pass

@defaultconfig
class JsonRPC(Protocol):
    '''
    JSON-RPC 1.0 Protocol
    '''
    _default_persist = True
    # This is the OVSDB default port
    _default_defaultport = 6632
    _default_createqueue = True
    # Print debugging log
    _default_debugging = False
    # JSON encoding
    _default_encoding = 'utf-8'
    _default_buffersize = 65536
    # Default limit a JSON message to 16MB for security purpose
    _default_messagelimit = 16777216
    # Default limit JSON scan level to 1024 levels
    _default_levellimit = 1024
    # Limit the allowed request methods
    _default_allowedrequests = None
    _logger = logging.getLogger(__name__ + '.JsonRPC')
    def __init__(self):
        '''
        Constructor
        '''
        Protocol.__init__(self)
    def init(self, connection):
        for m in Protocol.init(self, connection):
            yield m
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority, JsonRPCRequestEvent.createMatcher(connection = connection), ('request', connection), self.messagequeuesize))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority, JsonRPCConnectionStateEvent.createMatcher(connection = connection), ('connstate', connection)))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority + 1, JsonRPCResponseEvent.createMatcher(connection = connection), ('response', connection), self.messagequeuesize))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority, JsonRPCNotificationEvent.createMatcher(connection = connection), ('notification', connection), self.messagequeuesize))
        for m in self.reconnect_init(connection):
            yield m
    def reconnect_init(self, connection):
        connection.xid = ord(os.urandom(1)) + 1
        connection.jsonrpc_parserlevel = 0
        connection.jsonrpc_parserstate = 'begin'
        for m in connection.waitForSend(JsonRPCConnectionStateEvent(JsonRPCConnectionStateEvent.CONNECTION_UP, connection, connection.connmark, self)):
            yield m
    def closed(self, connection):
        for m in Protocol.closed(self, connection):
            yield m
        connection.scheduler.ignore(JsonRPCRequestEvent.createMatcher(connection = connection))
        self._logger.info('JSON-RPC connection is closed on %r', connection)
        for m in connection.waitForSend(JsonRPCConnectionStateEvent(JsonRPCConnectionStateEvent.CONNECTION_DOWN, connection, connection.connmark, self)):
            yield m
    def error(self, connection):
        for m in Protocol.error(self, connection):
            yield m
        connection.scheduler.ignore(JsonRPCRequestEvent.createMatcher(connection = connection))
        self._logger.warning('JSON-RPC connection is reset on %r', connection)
        for m in connection.waitForSend(JsonRPCConnectionStateEvent(JsonRPCConnectionStateEvent.CONNECTION_DOWN, connection, connection.connmark, self)):
            yield m
    _BEGIN_PATTERN = re.compile(br'\s*')
    _OBJECT_PATTERN = re.compile(br'[^"{}]*')
    _STRING_PATTERN = re.compile(br'[^"^\\]*')
    def formatrequest(self, method, params, connection):
        msgid = connection.xid
        msg = {'method': method, 'params': params, 'id': msgid}
        connection.xid += 1
        if connection.xid > 0x7fffffff:
            # Skip xid = 0 for special response
            connection.xid = 1
        c = ConnectionWriteEvent(connection = connection, connmark = connection.connmark, data = json.dumps(msg).encode(self.encoding))
        if self.debugging:
            self._logger.debug('message formatted: %r', msg)
        return (c, msgid)
    def formatnotification(self, method, params, connection):
        msg = {'method': method, 'params': params, 'id': None}
        c = ConnectionWriteEvent(connection = connection, connmark = connection.connmark, data = json.dumps(msg).encode(self.encoding))
        if self.debugging:
            self._logger.debug('message formatted: %r', msg)
        return c
    def formatreply(self, result, requestid, connection):
        msg = {'result': result, 'error': None, 'id': requestid}
        c = ConnectionWriteEvent(connection = connection, connmark = connection.connmark, data = json.dumps(msg).encode(self.encoding))
        if self.debugging:
            self._logger.debug('message formatted: %r', msg)
        return c
    def formaterror(self, error, requestid, connection):
        msg = {'result': None, 'error': error, 'id': requestid}
        c = ConnectionWriteEvent(connection = connection, connmark = connection.connmark, data = json.dumps(msg).encode(self.encoding))
        if self.debugging:
            self._logger.debug('message formatted: %r', msg)
        return c
    def replymatcher(self, requestid, connection, iserror = None):
        """
        Create a matcher to match a reply
        """
        matcherparam = {'connection' : connection, 'connmark': connection.connmark, 
                        'id': requestid}
        if iserror is not None:
            matcherparam['iserror'] = iserror
        return JsonRPCResponseEvent.createMatcher(**matcherparam)
    def notificationmatcher(self, method, connection):
        """
        Create an event matcher to match specified notifications
        """
        return JsonRPCNotificationEvent.createMatcher(method = method, connection = connection, connmark = connection.connmark)
    def statematcher(self, connection, state = JsonRPCConnectionStateEvent.CONNECTION_DOWN, currentconn = True):
        """
        Create an event matcher to match the connection state
        """
        if currentconn:
            return JsonRPCConnectionStateEvent.createMatcher(state, connection, connection.connmark)
        else:
            return JsonRPCConnectionStateEvent.createMatcher(state, connection)
    def querywithreply(self, method, params, connection, container, raiseonerror = True):
        """
        Send a JSON-RPC request and wait for the reply. The reply result is stored at
        `container.jsonrpc_result` and the reply error is stored at `container.jsonrpc_error`.
        """
        (c, rid) = self.formatrequest(method, params, connection)
        for m in connection.write(c, False):
            yield m
        reply = self.replymatcher(rid, connection)
        conndown = self.statematcher(connection)
        yield (reply, conndown)
        if container.matcher is conndown:
            raise JsonRPCProtocolException('Connection is down before reply received')
        container.jsonrpc_result = container.event.result
        container.jsonrpc_error = container.event.error
        if raiseonerror and container.event.error:
            raise JsonRPCErrorResultException(str(container.event.error))
    def waitfornotify(self, method, connection, container):
        """
        Wait for next notification
        """
        notify = self.notificationmatcher(method, connection)
        conndown = self.statematcher(connection)
        yield (notify, conndown)
        if container.matcher is conndown:
            raise JsonRPCProtocolException('Connection is down before notification received')
        container.jsonrpc_notifymethod = container.event.method
        container.jsonrpc_notifyparams = container.event.params
    def parse(self, connection, data, laststart):
        jsonstart = 0
        start = laststart
        end = len(data)
        events = []
        level = connection.jsonrpc_parserlevel
        state = connection.jsonrpc_parserstate
        _OBJECT_START = b'{'[0]
        _STRING_MARK = b'"'[0]
        _ESCAPE_MARK = b'\\'[0]
        _OBJECT_END = b'}'[0]
        while start < end:
            # We only match {} to find the end position
            if state == 'begin':
                m = self._BEGIN_PATTERN.match(data, start)
                start = m.end()
                if start < end:
                    if data[start] == _OBJECT_START:
                        start += 1
                        level += 1
                        state = 'object'
                    else:
                        raise JsonFormatException('"{" is not found')
            elif state == 'object':
                m = self._OBJECT_PATTERN.match(data, start)
                start = m.end()
                if start < end:
                    if data[start] == _STRING_MARK:
                        start += 1
                        state = 'string'
                    elif data[start] == _OBJECT_START:
                        start += 1
                        level += 1
                    elif data[start] == _OBJECT_END:
                        start += 1
                        level -= 1
                        if level <= 0:
                            state = 'begin'
                            jsondata = data[jsonstart:start]
                            if hasattr(jsondata, 'tobytes'):
                                jsondata = jsondata.tobytes()
                            jsondata = jsondata.decode(self.encoding)
                            if self.debugging:
                                self._logger.debug('Parsing json text:\n%s', jsondata)
                            jsondata = json.loads(jsondata)
                            if 'method' in jsondata:
                                if jsondata['method'] is None:
                                    raise JsonFormatException('method is None in input json')
                                if jsondata['id'] is not None:
                                    # Unprocessed requests will block the JSON-RPC connection message queue,
                                    # as a security consideration, the parser can automatically reject unknown
                                    # requests
                                    if self.allowedrequests is not None and str(jsondata['method']) not in self.allowedrequests:
                                        events.append(self.formaterror('method is not supported', jsondata['id'], connection))
                                    else:
                                        events.append(JsonRPCRequestEvent(method = str(jsondata['method']), params = jsondata['params'],
                                                                          id = jsondata['id'], connection = connection, connmark = connection.connmark, createby = self))
                                        self._logger.debug('Request received(method = %r, id = %r, connection = %r)', jsondata['method'], jsondata['id'], connection)
                                else:
                                    events.append(JsonRPCNotificationEvent(method = str(jsondata['method']), params = jsondata['params'],
                                                                      connection = connection, connmark = connection.connmark, createby = self))
                                    self._logger.debug('Notification received(method = %r, connection = %r)', str(jsondata['method']), connection)
                            elif 'result' in jsondata:
                                if jsondata['id'] is None:
                                    raise JsonFormatException('id is None for a response')
                                events.append(JsonRPCResponseEvent(connection = connection, connmark = connection.connmark,
                                                                   id = jsondata['id'], iserror = jsondata['error'] is not None,
                                                                   result = jsondata['result'], error = jsondata['error'], createby = self))
                                self._logger.debug('Response received(id = %r, connection = %r)', jsondata['id'], connection)
                            jsonstart = start
                    else:
                        # Never really reach
                        raise JsonFormatException('How can this be reached...')
            elif state == 'string':
                m = self._STRING_PATTERN.match(data, start)
                start = m.end()
                if start < end:
                    if data[start] == _STRING_MARK:
                        start += 1
                        state = 'object'
                    elif data[start] == _ESCAPE_MARK:
                        start += 1
                        state = 'escape'
                    else:
                        # Never really reach
                        raise JsonFormatException('How can this be reached...')
            else:
                # Escape
                start += 1
                state = 'string'
            # Security check
            if start - jsonstart > self.messagelimit:
                raise JsonFormatException('JSON message size exceeds limit')
            if level > self.levellimit:
                raise JsonFormatException('JSON message level exceeds limit')
        connection.jsonrpc_parserlevel = level
        connection.jsonrpc_parserstate = state
        if laststart == len(data):
            # Remote write close
            events.append(ConnectionWriteEvent(connection, connection.connmark, data = b'', EOF = True))
        return (events, len(data) - jsonstart)
    



'''
Created on 2016/10/20

:author: hubo
'''
from __future__ import print_function
from vlcp.server.module import Module
from vlcp.server import main as server_main
from vlcp.event.runnable import RoutineContainer
import sys
import getopt

class ScriptModule(Module):
    '''
    Base script module
    '''
    configkey = 'main'
    # This is not meant to be configured in a configuration file; this is done by the command line executor
    _default_args = ()
    # This is not meant to be configured in a configuration file; this is done by the command line executor
    _default_kwargs = {}
    options = ()
    def __init__(self, server):
        Module.__init__(self, server)
        self.apiroutine = RoutineContainer(self.scheduler)
        def _main():
            try:
                for m in self.run(*self.args, **self.kwargs):
                    yield m
            finally:
                self.scheduler.quit()
        self.apiroutine.main = _main
        self.routines.append(self.apiroutine)
    def run(self, *argv, **kwargs):
        print('Running script...')
        if False:
            yield
    @classmethod
    def main(cls):
        short_opts = 'f:h?'
        long_opts = ['help']
        short_dict = {}
        for opt in cls.options:
            hasarg = len(opt) < 3 or opt[2]
            if hasarg:
                if len(opt) > 1 and opt[1]:
                    short_opts += opt[1] + ':'
                    short_dict[opt[1]] = opt[0]
                long_opts.append(opt[0] + '=')
            else:
                if len(opt) > 1 and opt[1]:
                    short_opts += opt[1]
                    short_dict[opt[1]] = opt[0]
                long_opts.append(opt[0])
        try:
            options, args = getopt.gnu_getopt(sys.argv[1:], short_opts, long_opts)
        except Exception as exc:
            print(str(exc))
            print()
            print(cls.__doc__)
            sys.exit(2)
        else:
            opt_dict = {}
            configfile = None
            for k,v in options:
                if k == '--help' or k == '-?' or k == '-h':
                    print(cls.__doc__)
                    sys.exit(0)
                elif k == '-f':
                    configfile = v
                else:
                    if k.startswith('--'):
                        opt_dict[k[2:]] = v
                    else:
                        opt_dict[short_dict[k[1:]]] = v
            from vlcp.config import manager
            manager['main.args'] = args
            manager['main.kwargs'] = opt_dict
            server_main(configfile, ('__main__.' + cls.__name__,))



'''
Created on 2016/12/1

:author: hubo
'''
from vlcp.config import defaultconfig
from vlcp.server.module import Module, api, depend, ModuleNotification, callAPI
from vlcp.event.runnable import RoutineContainer
import vlcp.utils.ovsdb as ovsdb
import vlcp.service.connection.jsonrpcserver as jsonrpcserver
from vlcp.protocol.jsonrpc import JsonRPCErrorResultException,\
    JsonRPCConnectionStateEvent, JsonRPCNotificationEvent
from vlcp.event.event import Event, withIndices
from contextlib import closing
from vlcp.event.connection import ConnectionResetException
from vlcp.utils.vxlandiscover import lognet_vxlan_walker, update_vxlaninfo,\
    get_broadcast_ips
import vlcp.service.kvdb.objectdb as objectdb
from vlcp.utils.networkmodel import LogicalNetwork, VXLANEndpointSet,\
    LogicalNetworkMap, LogicalPortVXLANInfo, LogicalPort
from vlcp.utils.dataobject import multiwaitif
from vlcp.utils.ethernet import ip4_addr

@withIndices('connection')
class VtepConnectionSynchronized(Event):
    pass


@withIndices('state', 'name', 'connection', 'connmark', 'vhost')
class VtepPhysicalSwitchStateChanged(Event):
    UP = 'up'
    DOWN = 'down'


@withIndices('identifier')
class _DataUpdateEvent(Event):
    pass

def _check_transact_result(result, operations):
    if any('error' in r for r in result if r is not None):
        err_info = next((r['error'],
                     operations[i] if i < len(operations) else None)
                    for i,r in enumerate(result)
                    if r is not None and 'error' in r)
        raise JsonRPCErrorResultException('Error in OVSDB select operation: %r. Corresponding operation: %r' % err_info)

@defaultconfig
@depend(jsonrpcserver.JsonRPCServer, objectdb.ObjectDB)
class VtepController(Module):
    """
    Controll a physical switch which supports OVSDB hardware_vtep protocol.
    """
    # Default bind controller to OVSDB vHost
    _default_vhostbind = ['vtep']
    # Recycle unused logical switches from hardware_vtep OVSDB
    _default_recycleinterval = 300
    # Every logical network has a broadcast list which contains all related server nodes,
    # this is the refresh interval to keep current node in the list
    _default_refreshinterval = 3600
    # When a logical port is migrating from one node to another, there may be multiple
    # instances of the logical port on different or same nodes. This is the maximum time
    # allowed for the migration.
    _default_allowedmigrationtime = 120
    # When there are multiple controllers started for the same hardware_vtep OVSDB,
    # every instance will try to acquire this lock in the OVSDB, and the one who
    # acquired the lock will be the master. 
    _default_masterlock = "vlcp_vtepcontroller_masterlock"
    # Prepush unicast information for MACs on other nodes. If this is not set, the
    # physical switch should support and enable MAC-learning on VXLAN.
    _default_prepush = True
    def __init__(self, server):
        Module.__init__(self, server)
        self.apiroutine = RoutineContainer(self.scheduler)
        self.apiroutine.main = self._main
        self.routines.append(self.apiroutine)
        self.createAPI(api(self.listphysicalports, self.apiroutine),
                       api(self.listphysicalswitches, self.apiroutine),
                       api(self.updatelogicalswitch, self.apiroutine),
                       api(self.unbindlogicalswitch, self.apiroutine),
                       api(self.unbindphysicalport, self.apiroutine))
        self._physical_switchs = {}
        self._connection_ps = {}
        self._synchronized = False
        self._monitor_routines = set()
        self._requestid = 0
        
    def _wait_for_sync(self):
        if not self._synchronized:
            yield (ModuleNotification.createMatcher(self.getServiceName(), 'synchronized'),)
    
    def _monitor_conn(self, conn, notification = False):
        current_routine = self.apiroutine.currentroutine
        self._monitor_routines.add(current_routine)
        ls_monitor = self.apiroutine.subroutine(self._monitor_logicalswitches(conn))
        initialized = False
        protocol = conn.protocol
        def _process_result(result):
            if 'old' in result:
                # Removed or modified
                if 'name' in result['old']:
                    oldname = result['old']['name']
                else:
                    oldname = result['new']['name']
                try:
                    del self._physical_switchs[oldname]
                except KeyError:
                    pass
                try:
                    self._connection_ps[conn].discard(oldname)
                except KeyError:
                    pass
                if 'new' not in result or 'name' in result['old']:
                    self.scheduler.emergesend(VtepPhysicalSwitchStateChanged(VtepPhysicalSwitchStateChanged.DOWN,
                                                                             oldname,
                                                                             conn,
                                                                             conn.connmark,
                                                                             protocol.vhost
                                                                             ))
            if 'new' in result:
                # Created or modified
                nr = result['new']
                self._connection_ps.setdefault(conn, set()).add(nr['name'])
                tunnel_ips = ovsdb.getlist(nr['tunnel_ips'])
                if not tunnel_ips:
                    self._logger.warning('Physical switch %r does not have a configured tunnel IP', )
                    tunnelip = None
                else:
                    tunnelip = tunnel_ips[0]
                self._physical_switchs[nr['name']] = (conn, tunnelip)
                if 'old' not in result or 'name' in result['old']:
                    self.scheduler.emergesend(VtepPhysicalSwitchStateChanged(VtepPhysicalSwitchStateChanged.UP,
                                                                             nr['name'],
                                                                             conn,
                                                                             conn.connmark,
                                                                             protocol.vhost
                                                                             ))                
        try:
            method, params = ovsdb.monitor(
                                'hardware_vtep',
                                'vlcp_vtepcontroller_physicalswitch_monitor',
                                {
                                    'Physical_Switch':
                                        ovsdb.monitor_request(['name', 'tunnel_ips'], True, True, True, True)
                                })
            try:
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
            except JsonRPCErrorResultException:
                # The monitor is already set, cancel it first
                method2, params2 = ovsdb.monitor_cancel('vlcp_vtepcontroller_physicalswitch_monitor')
                for m in protocol.querywithreply(method2, params2, conn, self.apiroutine, False):
                    yield m
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
            for r in result['Physical_Switch'].values():
                _process_result(r)
            initialized = True
            if notification:
                self.scheduler.emergesend(VtepConnectionSynchronized(conn))
            monitor_matcher = ovsdb.monitor_matcher(conn, 'vlcp_vtepcontroller_physicalswitch_monitor')
            conn_state = protocol.statematcher(conn)
            while True:
                yield (monitor_matcher, conn_state)
                if self.apiroutine.matcher is conn_state:
                    break
                else:
                    for r in self.apiroutine.event.params[1]['Physical_Switch'].values():
                        _process_result(r)
        except Exception:
            self._logger.warning('Initialize OVSDB vtep connection failed, maybe this is not a valid vtep endpoint',
                                 exc_info = True)
        finally:
            ls_monitor.close()
            if notification and not initialized:
                self.scheduler.emergesend(VtepConnectionSynchronized(conn))
            if conn in self._connection_ps:
                ps_set = self._connection_ps[conn]
                for ps in ps_set:
                    if ps in self._physical_switchs:
                        del self._physical_switchs[ps]
                del self._connection_ps[conn]
            self._monitor_routines.discard(current_routine)

    def _monitor_logicalswitches(self, conn):
        protocol = conn.protocol
        lockid = self.masterlock
        # There may be multiple vtep controllers running, only the one which successfully acquired the
        # master lock should update the ObjectDB
        method, params = ovsdb.lock(lockid)
        def _unlock():
            method, params = ovsdb.unlock(lockid)
            if conn.connected:
                try:
                    for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                        yield m
                except Exception:
                    # Any result is acceptable, including: error result; connection down; etc.
                    # We don't report an exception that we cannot do anything for it.
                    pass
        try:
            for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                yield m
        except JsonRPCErrorResultException:
            # For some reason the lock is not released? let's try it out
            for m in _unlock():
                yield m
            for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                yield m                
        result = self.apiroutine.jsonrpc_result
        switch_routines = {}
        try:
            if not result['locked']:
                # Wait for lock
                lock_notification = JsonRPCNotificationEvent.createMatcher(
                                            "locked",
                                            conn,
                                            conn.connmark,
                                            _ismatch = lambda x: x.params[0] == lockid)
                conn_matcher = protocol.statematcher(conn)
                yield (lock_notification, conn_matcher)
                if self.apiroutine.matcher is conn_matcher:
                    return
            # Now we acquired the lock, start monitoring the logical switches
            method, params = ovsdb.monitor(
                                'hardware_vtep',
                                'vlcp_vtepcontroller_logicalswitch_monitor',
                                {
                                    'Logical_Switch':
                                        ovsdb.monitor_request(['name'], True, True, True, False)
                                })
            try:
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
            except JsonRPCErrorResultException:
                # The monitor is already set, cancel it first
                method2, params2 = ovsdb.monitor_cancel('vlcp_vtepcontroller_logicalswitch_monitor')
                for m in protocol.querywithreply(method2, params2, conn, self.apiroutine, False):
                    yield m
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
            if 'Logical_Switch' in result:
                for k, r in result['Logical_Switch'].items():
                    switch_routines[k] = self.apiroutine.subroutine(self._update_logicalswitch_info(conn, r['new']['name'], k))
            monitor_matcher = ovsdb.monitor_matcher(conn, 'vlcp_vtepcontroller_logicalswitch_monitor')
            conn_state = protocol.statematcher(conn)
            while True:
                yield (monitor_matcher, conn_state)
                if self.apiroutine.matcher is conn_state:
                    break
                updates = self.apiroutine.event.params[1]
                if 'Logical_Switch' in updates:
                    for k,r in self.apiroutine.event.params[1]['Logical_Switch'].items():
                        if 'old' in r:
                            if k in switch_routines:
                                switch_routines[k].close()
                                del switch_routines[k]
                        else:
                            if k in switch_routines:
                                switch_routines[k].close()
                            switch_routines[k] = self.apiroutine.subroutine(self._update_logicalswitch_info(conn, r['new']['name'], k))
        finally:
            for r in switch_routines.values():
                r.close()
            self.apiroutine.subroutine(_unlock(), False)
    
    def _push_logicalswitch_endpoints(self, conn, logicalswitchid):
        while True:
            if conn in self._connection_ps:
                ps_list = [(name, self._physical_switchs[name][1])
                           for name in self._connection_ps[conn]]
                for name, tunnel_ip in ps_list:
                    with closing(update_vxlaninfo(
                                    self.apiroutine,
                                    {logicalswitchid: tunnel_ip},
                                    {},
                                    {},
                                    conn.protocol.vhost,
                                    name,
                                    'hardware_vtep',
                                    self.allowedmigrationtime,
                                    self.refreshinterval)) as g:
                        for m in g:
                            yield m
            for m in self.apiroutine.waitWithTimeout(self.refreshinterval):
                yield m
    
    def _poll_logicalswitch_endpoints(self, conn, logicalswitchid, lsuuid):
        protocol = conn.protocol
        walker = lognet_vxlan_walker(self.prepush)
        self._requestid += 1
        requestid = ('vtepcontroller', self._requestid)
        lognet_key = LogicalNetwork.default_key(logicalswitchid)
        endpointset_key = VXLANEndpointSet.default_key(logicalswitchid)
        lognetmap_key = LogicalNetworkMap.default_key(logicalswitchid)
        rewalk_keys = (lognet_key, endpointset_key, lognetmap_key)
        _update_set = set()
        _detect_update_routine = None
        identifier = object()
        def _detect_update(saved_result):
            while True:
                for m in multiwaitif(saved_result, self.apiroutine, lambda x,y: True, True):
                    yield m
                updated_values, _ = self.apiroutine.retvalue
                if not _update_set:
                    self.apiroutine.scheduler.emergesend(_DataUpdateEvent(identifier))
                _update_set.update(updated_values)
        update_matcher = _DataUpdateEvent.createMatcher(identifier)
        _savedkeys = ()
        _savedresult = ()
        # These are: set of network tunnelips; dictionary for mac: tunnelip; last_physical_switch
        _last_endpoints = [set(), {}, None]
        
        def _update_hardware_vtep(saved_result):
            if conn not in self._connection_ps or not self._connection_ps[conn]:
                _last_endpoints[:] = [set(), {}, None]
                return
            # There should be only one physical switch in this connection
            ps = next(iter(self._connection_ps[conn]))
            allobjs = [v for v in saved_result if v is not None and not v.isdeleted()]
            lognet_list = [o for o in allobjs if o.getkey() == lognet_key]
            if not lognet_list or not hasattr(lognet_list[0], 'vni'):
                # Logical network maybe deleted, or not a VXLAN network, do not need to update
                return
            vni = lognet_list[0].vni
            endpointset_list = [v for v in allobjs if v.getkey() == endpointset_key]
            local_ip = self._physical_switchs[ps][1]
            if endpointset_list:
                broadcast_ips = set(ip for ip,_ in get_broadcast_ips(endpointset_list[0], local_ip,
                                                  protocol.vhost,
                                                  ps,
                                                  'hardware_vtep'))
            else:
                broadcast_ips = set()
            if self.prepush:
                logport_tunnel_ip_dict = dict((o.id, o.endpoints[0]['tunnel_dst'])
                                              for o in allobjs
                                              if o.isinstance(LogicalPortVXLANInfo) \
                                              and hasattr(o, 'endpoints') and o.endpoints \
                                              and (o.endpoints[0]['vhost'],
                                                   o.endpoints[0]['systemid'],
                                                   o.endpoints[0]['bridge'])
                                                != (protocol.vhost, ps, 'hardware_vtep'))
                mac_ip_dict = dict((p.mac_address, logport_tunnel_ip_dict[p.id])
                                   for p in allobjs
                                   if p.isinstance(LogicalPort) \
                                   and hasattr(p, 'mac_address') \
                                   and p.id in logport_tunnel_ip_dict)
            else:
                mac_ip_dict = {}
            if _last_endpoints[2] == ps:
                # Update
                update = True
                broadcasts_updated = (broadcast_ips != _last_endpoints[0])
                add_ucasts = dict((m, mac_ip_dict[m])
                                  for m in mac_ip_dict
                                  if m not in _last_endpoints[1] or \
                                    _last_endpoints[1][m] != mac_ip_dict[m])
                remove_ucasts = dict((m, _last_endpoints[1][m])
                                     for m in _last_endpoints[1]
                                     if m not in mac_ip_dict)
            else:
                update = False
                broadcasts_updated = True
                remove_broadcasts = set()
                add_ucasts = mac_ip_dict
                remove_ucasts = {}
            using_ips = list(broadcast_ips.union(add_ucasts.values()))
            try:
                while True:
                    # Check and set
                    check_operates = [ovsdb.select('Logical_Switch',
                                                     [["_uuid", "==", ovsdb.uuid(lsuuid)],
                                                      ["name", "==", logicalswitchid]],
                                                     ["_uuid", "name", "tunnel_key"]),
                                      ovsdb.select('Mcast_Macs_Remote',
                                                     [["MAC", "==", "unknown-dst"],
                                                      ["logical_switch", "==", ovsdb.uuid(lsuuid)]],
                                                     ["_uuid", "locator_set"])]
                    # Check if each destination location exists
                    check_operates.extend(ovsdb.select('Physical_Locator',
                                                       [["dst_ip", "==", ip],
                                                        ["encapsulation_type", "==", "vxlan_over_ipv4"]],
                                                       ["_uuid",  "dst_ip"])
                                          for ip in using_ips)
                    method, params = ovsdb.transact('hardware_vtep',
                                                    *check_operates)
                    for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                        yield m
                    result = self.apiroutine.jsonrpc_result
                    _check_transact_result(result, check_operates)
                    if not result[0]['rows']:
                        # Logical switch is removed
                        break
                    wait_operates = [ovsdb.wait('Logical_Switch',
                                               [["_uuid", "==", ovsdb.uuid(lsuuid)]],
                                               ["name"],
                                               [{"name": logicalswitchid}], True, 0)]
                    set_operates = []
                    # Check destinations
                    locator_uuid_dict = {}
                    if result[0]['rows'][0]['tunnel_key'] != vni:
                        set_operates.append(ovsdb.update('Logical_Switch',
                                               [["_uuid", "==", ovsdb.uuid(lsuuid)]],
                                               {"tunnel_key": vni}))
                    for r,ip in zip(result[2:len(check_operates)], using_ips):
                        if r['rows']:
                            locator = r['rows'][0]
                            # This locator is already created
                            locator_uuid_dict[ip] = locator['_uuid']
                            wait_operates.append(ovsdb.wait('Physical_Locator',
                                                       [["_uuid", "==", locator['_uuid']]],
                                                       ["dst_ip", "encapsulation_type"],
                                                       [{"dst_ip": ip,
                                                         "encapsulation_type": "vxlan_over_ipv4"}],
                                                        True, 0))
                        else:
                            # Create the locator
                            wait_operates.append(ovsdb.wait('Physical_Locator',
                                                            [["dst_ip", "==", ip],
                                                             ["encapsulation_type", "==", "vxlan_over_ipv4"]],
                                                            ["_uuid"],
                                                            [], True, 0))
                            name = 'locator_uuid_' + hex(ip4_addr(ip))
                            locator_uuid_dict[ip] = ovsdb.named_uuid(name)
                            set_operates.append(ovsdb.insert('Physical_Locator',
                                                             {"dst_ip": ip,
                                                              "encapsulation_type": "vxlan_over_ipv4"},
                                                             name))
                    if result[1]['rows'] and (not update or broadcasts_updated):
                        # locator set cannot be modified, it can only be replaced with a new set
                        mcast_uuid = result[1]['rows'][0]['locator_set']
                        # locator set is already created
                        if not broadcast_ips:
                            # Remove the whole locator set. We do not do check; it is always removed
                            set_operates.append(ovsdb.delete('Mcast_Macs_Remote',
                                                             [["MAC", "==", "unknown-dst"],
                                                              ["logical_switch", "==", ovsdb.uuid(lsuuid)]]))
                        else:
                            wait_operates.append(ovsdb.wait('Mcast_Macs_Remote',
                                                          [["MAC", "==", "unknown-dst"],
                                                          ["logical_switch", "==", ovsdb.uuid(lsuuid)]],
                                                          ["_uuid"],
                                                          [{"_uuid": result[1]['rows'][0]['_uuid']}],
                                                          True, 0))
                            # Create a new set
                            set_operates.append(ovsdb.insert('Physical_Locator_Set',
                                                             {"locators": ovsdb.oset(*[locator_uuid_dict[ip] for ip in broadcast_ips])},
                                                             'mcast_unknown_uuid'))
                            mcast_uuid = ovsdb.named_uuid('mcast_unknown_uuid')
                            set_operates.append(ovsdb.update('Mcast_Macs_Remote',
                                                            [["_uuid", "==", result[1]['rows'][0]['_uuid']]],
                                                             {"locator_set": mcast_uuid}))
                    else:
                        if broadcast_ips:
                            wait_operates.append(ovsdb.wait('Mcast_Macs_Remote',
                                                          [["MAC", "==", "unknown-dst"],
                                                          ["logical_switch", "==", ovsdb.uuid(lsuuid)]],
                                                          ["_uuid"],
                                                          [],
                                                          True, 0))
                            set_operates.append(ovsdb.insert('Physical_Locator_Set',
                                                             {"locators": ovsdb.oset(*[locator_uuid_dict[ip] for ip in broadcast_ips])},
                                                             'mcast_unknown_uuid'))
                            mcast_uuid = ovsdb.named_uuid('mcast_unknown_uuid')
                            set_operates.append(ovsdb.insert('Mcast_Macs_Remote',
                                                             {"MAC": "unknown-dst",
                                                              "logical_switch": ovsdb.uuid(lsuuid),
                                                              "locator_set": mcast_uuid}))
                    # modify unicast table
                    if update:
                        set_operates.extend(ovsdb.delete('Ucast_Macs_Remote',
                                                         [["logical_switch", "==", ovsdb.uuid(lsuuid)],
                                                          ["MAC", "==", mac]])
                                            for mac in remove_ucasts)
                        set_operates.extend(ovsdb.delete('Ucast_Macs_Remote',
                                                         [["logical_switch", "==", ovsdb.uuid(lsuuid)],
                                                          ["MAC", "==", mac]])
                                            for mac in add_ucasts)
                        set_operates.extend(ovsdb.insert('Ucast_Macs_Remote',
                                                         {"MAC": mac,
                                                          "logical_switch": ovsdb.uuid(lsuuid),
                                                          "locator": locator_uuid_dict[tunnel]})
                                            for mac, tunnel in add_ucasts.items())
                    else:
                        set_operates.append(ovsdb.delete('Ucast_Macs_Remote',
                                                         [["logical_switch", "==", ovsdb.uuid(lsuuid)]]))
                        set_operates.extend(ovsdb.insert('Ucast_Macs_Remote',
                                                         {"MAC": mac,
                                                          "logical_switch": ovsdb.uuid(lsuuid),
                                                          "locator": locator_uuid_dict[tunnel]})
                                            for mac, tunnel in add_ucasts.items())
                    method, params = ovsdb.transact('hardware_vtep', *(wait_operates + set_operates))
                    for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                        yield m
                    result = self.apiroutine.jsonrpc_result
                    if any(r is None or 'error' in r for r in result[:len(wait_operates)]):
                        # Some wait operates failed, retry
                        continue
                    _check_transact_result(result[len(wait_operates):], set_operates)
                    break
            except Exception:
                self._logger.warning('Update hardware_vtep failed with exception', exc_info = True)
                # Force a full update next time
                _last_endpoints[:] = [set(), {}, None]
            else:
                _last_endpoints[:] = [broadcast_ips, mac_ip_dict, ps]
        try:
            while True:
                for m in callAPI(self.apiroutine, 'objectdb', 'walk',
                                 {'keys': (lognet_key,
                                           endpointset_key,
                                           lognetmap_key),
                                  'walkerdict': {lognet_key: walker},
                                  'requestid': requestid
                                  }):
                    yield m
                if _update_set:
                    _update_set.clear()
                    continue
                lastkeys = set(_savedkeys)
                _savedkeys, _savedresult = self.apiroutine.retvalue
                if _detect_update_routine is not None:
                    _detect_update_routine.close()
                _detect_update_routine = self.apiroutine.subroutine(_detect_update(_savedresult), False)
                removekeys = tuple(lastkeys.difference(_savedkeys))
                if removekeys:
                    # Unwatch unnecessary keys
                    for m in callAPI(self.apiroutine, 'objectdb', 'munwatch', {'keys': removekeys,
                                                                    'requestid': requestid}):
                        yield m
                for m in _update_hardware_vtep(_savedresult):
                    yield m
                while True:
                    if not _update_set:
                        yield (update_matcher,)
                    should_rewalk = any(v.getkey() in rewalk_keys for v in _update_set if v is not None)
                    _update_set.clear()
                    if should_rewalk:
                        break
                    else:
                        for m in _update_hardware_vtep(_savedresult):
                            yield m
        finally:
            if _detect_update_routine is not None:
                _detect_update_routine.close()
            self.apiroutine.subroutine(callAPI(self.apiroutine,'objectdb', 'unwatchall', {'requestid': requestid}))
    
    def _update_logicalswitch_info(self, conn, logicalswitch, lsuuid):
        with closing(self.apiroutine.executeAll([self._push_logicalswitch_endpoints(conn, logicalswitch),
                                                 self._poll_logicalswitch_endpoints(conn, logicalswitch, lsuuid)], None, ())) as g:
            for m in g:
                yield m
    
    def _manage_existing(self):
        for m in callAPI(self.apiroutine, "jsonrpcserver", "getconnections", {}):
            yield m
        vb = self.vhostbind
        conns = self.apiroutine.retvalue
        matchers = []
        for c in conns:
            if vb is None or c.protocol.vhost in vb:
                self.apiroutine.subroutine(self._monitor_conn(c, True))
                matchers.append(VtepConnectionSynchronized.createMatcher(c))
        for m in self.apiroutine.waitForAll(*matchers):
            yield m
        self._synchronized = True
        for m in self.apiroutine.waitForSend(ModuleNotification(self.getServiceName(), 'synchronized')):
            yield m
            
    def _recycling(self):
        interval = self.recycleinterval
        if interval is None:
            return
        def _recycle_logical_switches(conn):
            try:
                protocol = conn.protocol
                method, params = ovsdb.transact('hardware_vtep',
                                                ovsdb.select('Logical_Switch',
                                                             [],
                                                             ["_uuid", "name"]))
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                ls_list = [(r['_uuid'][1], r['name']) for r in self.apiroutine.jsonrpc_result[0]['rows']]
                for l, name in ls_list:
                    method, params = ovsdb.transact('hardware_vtep',
                                                    ovsdb.delete('Ucast_Macs_Remote',
                                                                 [["logical_switch", "==", ovsdb.uuid(l)]]),
                                                    ovsdb.delete('Mcast_Macs_Remote',
                                                                 [["logical_switch", "==", ovsdb.uuid(l)]]),
                                                    ovsdb.delete('Ucast_Macs_Local',
                                                                 [["logical_switch", "==", ovsdb.uuid(l)]]),
                                                    ovsdb.delete('Mcast_Macs_Local',
                                                                 [["logical_switch", "==", ovsdb.uuid(l)]]),
                                                    ovsdb.delete('Logical_Switch',
                                                                 [["_uuid", "==", ovsdb.uuid(l)]]))
                    for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                        yield m
                    # If the logical switch is still referenced, the transact will fail
                    if 'error' in self.apiroutine.jsonrpc_result[0]:
                        self._logger.warning('Recycling logical switch %r (uuid = %r) failed: %r',
                                             name, l, self.apiroutine.jsonrpc_result[0]['error'])
                    elif not any(r for r in self.apiroutine.jsonrpc_result if 'error' in r):
                        self._logger.info('Recycle logical switch %r for no longer in use', name)
            except Exception:
                self._logger.warning('Recycling failed on connecton %r', conn, exc_info = True)
        while True:
            for m in self.apiroutine.waitWithTimeout(interval):
                yield m
            with closing(self.apiroutine.executeAll([_recycle_logical_switches(c)
                                                     for c in self._connection_ps], None, ())) as g:
                for m in g:
                    yield m
                    
    def _main(self):
        self.apiroutine.subroutine(self._manage_existing())
        self.apiroutine.subroutine(self._recycling(), True, '_recycling_routine')
        try:
            vb = self.vhostbind
            if vb is not None:
                conn_up = JsonRPCConnectionStateEvent.createMatcher(state = JsonRPCConnectionStateEvent.CONNECTION_UP,
                                                                     _ismatch = lambda x: x.createby.vhost in vb)
            else:
                conn_up = JsonRPCConnectionStateEvent.createMatcher(state = JsonRPCConnectionStateEvent.CONNECTION_UP)
            while True:
                yield (conn_up,)
                self.apiroutine.subroutine(self._monitor_conn(self.apiroutine.event.connection))
        finally:
            for r in list(self._monitor_routines):
                r.close()
            if hasattr(self.apiroutine, '_recycling_routine'):
                self.apiroutine._recycling_routine.close()
        
    def listphysicalports(self, physicalswitch = None):
        '''
        Get physical ports list from this controller, grouped by physical switch name
        
        :param physicalswitch: physicalswitch name. Return all switches if is None.
        
        :return: dictionary: {physicalswitch: [physicalports]} e.g. {'ps1': ['port1', 'port2']}
        '''
        for m in self._wait_for_sync():
            yield m
        def _getports(conn):
            try:
                method, params = ovsdb.transact('hardware_vtep',
                                                ovsdb.select('Physical_Switch', [], ["name", "ports"]),
                                                ovsdb.select('Physical_Port', [], ["_uuid", "name"]))
                for m in conn.protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
                if 'error' in result[0] or 'error' in result[1]:
                    raise JsonRPCErrorResultException('OVSDB request failed: ' + repr(result))
                switches = result[0]['rows']
                ports = result[1]['rows']
                ports_dict = dict((r['_uuid'][1], r['name']) for r in ports)
                self.apiroutine.retvalue = dict((switch['name'],
                                                 [ports_dict[p[1]] for p in ovsdb.getlist(switch['ports'])
                                                  if p[1] in ports_dict])
                                                for switch in switches)
            except Exception:
                self._logger.warning('Query OVSDB on %r failed with exception; will ignore this connection',
                                     conn, exc_info = True)
                self.apiroutine.retvalue = {}
        if physicalswitch is None:
            routines = [_getports(c) for c in self._connection_ps]
            with closing(self.apiroutine.executeAll(routines)) as g:
                for m in g:
                    yield m
            all_result = {}
            for (r,) in self.apiroutine.retvalue:
                all_result.update(r)
            self.apiroutine.retvalue = all_result
        else:
            if physicalswitch not in self._physical_switchs:
                self.apiroutine.retvalue = {}
            else:
                for m in _getports(self._physical_switchs[physicalswitch]):
                    yield m
                result = self.apiroutine.retvalue
                if physicalswitch in result:
                    self.apiroutine.retvalue = {physicalswitch: result[physicalswitch]}
                else:
                    self.apiroutine.retvalue = {}
    
    def listphysicalswitches(self, physicalswitch = None):
        '''
        Get physical switch info
        
        :param physicalswitch: physicalswitch name. Return all switches if is None.
        
        :return: dictionary: {physicalswitch: {key: value}} keys include: management_ips,
                             tunnel_ips, description, switch_fault_status
        '''
        for m in self._wait_for_sync():
            yield m
        def _getswitch(conn):
            try:
                method, params = ovsdb.transact('hardware_vtep',
                                                ovsdb.select('Physical_Switch', [],
                                                             ["name", "management_ips", "tunnel_ips",
                                                              "description", "switch_fault_status"]))
                for m in conn.protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
                if 'error' in result[0]:
                    raise JsonRPCErrorResultException('OVSDB request failed: ' + repr(result))
                switches = result[0]['rows']
                self.apiroutine.retvalue = dict((s['name'],
                                                 {'management_ips': ovsdb.getlist(s['management_ips']),
                                                  'tunnel_ips': ovsdb.getlist(s['tunnel_ips']),
                                                  'description': s['description'],
                                                  'switch_fault_status': ovsdb.getlist(s['switch_fault_status'])})
                                                for s in switches)
            except Exception:
                self._logger.warning('Query OVSDB on %r failed with exception; will ignore this connection',
                                     conn, exc_info = True)
                self.apiroutine.retvalue = {}
        if physicalswitch is None:
            routines = [_getswitch(c) for c in self._connection_ps]
            with closing(self.apiroutine.executeAll(routines)) as g:
                for m in g:
                    yield m
            all_result = {}
            for (r,) in self.apiroutine.retvalue:
                all_result.update(r)
            self.apiroutine.retvalue = all_result
        else:
            if physicalswitch not in self._physical_switchs:
                self.apiroutine.retvalue = {}
            else:
                for m in _getswitch(self._physical_switchs[physicalswitch]):
                    yield m
                result = self.apiroutine.retvalue
                if physicalswitch in result:
                    self.apiroutine.retvalue = {physicalswitch: result[physicalswitch]}
                else:
                    self.apiroutine.retvalue = {}
        
    def updatelogicalswitch(self, physicalswitch, physicalport, vlanid, logicalnetwork, vni, logicalports):
        '''
        Bind VLAN on physicalport to specified logical network, and update logical port vxlan info
        
        :param physicalswitch: physical switch name, should be the name in PhysicalSwitch table of OVSDB vtep database
        
        :param physicalport: physical port name, should be the name in OVSDB vtep database
        
        :param vlanid: the vlan tag used for this logicalswitch
        
        :param logicalnetwork: the logical network id, will also be the logical switch id
        
        :param vni: the VXLAN VNI of the logical network
        
        :param logicalports: a list of logical port IDs. The VXLAN info of these ports will be updated.
        '''
        for m in self._wait_for_sync():
            yield m
        vlanid = int(vlanid)
        vni = int(vni)
        if physicalswitch is None:
            raise ValueError('Physical switch cannot be None')
        # We may retry some times
        while True:
            if physicalswitch not in self._physical_switchs:
                for m in self.apiroutine.waitWithTimeout(5,
                                                         VtepPhysicalSwitchStateChanged.createMatcher(
                                                            VtepPhysicalSwitchStateChanged.UP,
                                                            physicalswitch)):
                    yield m
                if self.apiroutine.timeout or physicalswitch not in self._physical_switchs:
                    raise ValueError('Physical switch %r is not found' % (physicalswitch,))
            conn, tunnelip = self._physical_switchs[physicalswitch]
            protocol = conn.protocol
            # First use a select to validate the request
            try:
                method, params = ovsdb.transact('hardware_vtep',
                                                ovsdb.select('Physical_Switch',
                                                             [["name", "==", physicalswitch]],
                                                             ["_uuid", "name", "ports"]),
                                                ovsdb.select('Physical_Port',
                                                             [["name", "==", physicalport]],
                                                             ["_uuid", "name", "vlan_bindings"]),
                                                ovsdb.select('Logical_Switch',
                                                             [["name", "==", logicalnetwork]],
                                                             ["_uuid", "name"])
                                                )
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
                if 'rows' not in result[0]:
                    raise JsonRPCErrorResultException('select from Physical_Switch failed: ' + repr(result[0]))
                if 'rows' not in result[1]:
                    raise JsonRPCErrorResultException('select from Physical_Switch failed: ' + repr(result[1]))
                if 'rows' not in result[2]:
                    raise JsonRPCErrorResultException('select from Physical_Switch failed: ' + repr(result[2]))
                if not result[0]['rows']:
                    raise ValueError('Physical switch %r is not found, connection = %r' % (physicalswitch, conn))
                pswitch = result[0]['rows'][0]
                port_uuids = set(p[1] for p in ovsdb.getlist(pswitch['ports']))
                ports = [p for p in result[1]['rows'] if p['_uuid'][1] in port_uuids]
                if not ports:
                    raise ValueError('Physical port %r is not found in physical switch %r' % (physicalport, physicalswitch))
                port = ports[0]
                ls_list = result[2]['rows']
                if ls_list:
                    ls_uuid = ls_list[0]['_uuid'][1]
                    # Logical switch is already created
                    curr_network = ovsdb.omap_getvalue(port['vlan_bindings'], vlanid)
                    if curr_network is None:
                        # Not binded
                        binded = False
                        created = True
                    elif curr_network[1] == ls_uuid:
                        # Already binded
                        binded = True
                        created = True
                    else:
                        raise ValueError('VLAN tag %r on physical port %r (%r) already binds to other networks, uuid=%r' \
                                         % (vlanid, physicalport, physicalswitch, curr_network[1]))
                else:
                    # Logical switch is not created
                    binded = False
                    created = False
                    curr_network = ovsdb.omap_getvalue(port['vlan_bindings'], vlanid)
                    if curr_network is not None:
                        raise ValueError('VLAN tag %r on physical port %r (%r) already binds to other networks, uuid=%r' \
                                         % (vlanid, physicalport, physicalswitch, curr_network[1]))
                if binded:
                    break
                operations = []
                # First we must check that the situations are not changed
                operations.extend([ovsdb.wait("Physical_Switch",
                                              [["name", "==", physicalswitch]],
                                              ["_uuid"],
                                              [{"_uuid": pswitch['_uuid']}],
                                              timeout = 0),
                                   ovsdb.wait("Physical_Port",
                                              [["name", "==", physicalport]],
                                              ["_uuid"],
                                              [{"_uuid": port['_uuid']}],
                                              timeout = 0)])
                if created:
                    operations.append(ovsdb.wait("Logical_Switch",
                                                [["name", "==", logicalnetwork]],
                                                ["_uuid"],
                                                [{"_uuid": ovsdb.uuid(ls_uuid)}],
                                                timeout = 0))
                    operations.append(ovsdb.update("Logical_Switch",
                                                   [["_uuid", "==", ovsdb.uuid(ls_uuid)]],
                                                   {"tunnel_key": vni}))
                else:
                    operations.append(ovsdb.wait("Logical_Switch",
                                                [["name", "==", logicalnetwork]],
                                                ["_uuid"],
                                                [],
                                                timeout = 0))
                    # Create the new logical switch
                    operations.append(ovsdb.insert('Logical_Switch',
                                                   {"name": logicalnetwork,
                                                    "tunnel_key": vni},
                                                   "new_logicalnetwork"))
                operations.append(ovsdb.mutate('Physical_Port', [["_uuid", "==", port['_uuid']]],
                                               [["vlan_bindings","insert",
                                                    ovsdb.omap(
                                                        (vlanid, ovsdb.uuid(ls_uuid) if created
                                                                 else ovsdb.named_uuid('new_logicalnetwork'))
                                                    )]]))
                method, params = ovsdb.transact('hardware_vtep', *operations)
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
                if any(r for r in result[:3] if 'error' in r):
                    # Wait failed, the OVSDB is modified, retry
                    continue
                _check_transact_result(result[3:], operations[3:])
                break
            except ConnectionResetException:
                for m in self.apiroutine.waitWithTimeout(1):
                    yield m
                continue
            except IOError:
                for m in self.apiroutine.waitWithTimeout(1):
                    yield m
                continue
        if logicalports:
            # Refresh network with monitor, only update the logical port information
            for m in update_vxlaninfo(self.apiroutine,
                                      {},
                                      dict((p, tunnelip) for p in logicalports),
                                      {},
                                      protocol.vhost,
                                      physicalswitch,
                                      'hardware_vtep',
                                      self.allowedmigrationtime,
                                      self.refreshinterval):
                yield m
        self.apiroutine.retvalue = None
        
    def unbindlogicalswitch(self, physicalswitch, physicalport, vlanid, logicalnetwork):
        '''
        Remove bind of a physical port
        
        :param physicalswitch: physical switch name, should be the name in PhysicalSwitch table of OVSDB vtep database
        
        :param physicalport: physical port name, should be the name in OVSDB vtep database
        
        :param vlanid: the vlan tag used for this logicalswitch
        
        :param logicalnetwork: the logical network id, will also be the logical switch id
        '''
        for m in self._wait_for_sync():
            yield m
        vlanid = int(vlanid)
        if physicalswitch is None:
            raise ValueError('Physical switch cannot be None')
        # We may retry some times
        while True:
            if physicalswitch not in self._physical_switchs:
                for m in self.apiroutine.waitWithTimeout(5,
                                                         VtepPhysicalSwitchStateChanged.createMatcher(
                                                            VtepPhysicalSwitchStateChanged.UP,
                                                            physicalswitch)):
                    yield m
                if self.apiroutine.timeout or physicalswitch not in self._physical_switchs:
                    raise ValueError('Physical switch %r is not found' % (physicalswitch,))
            conn, tunnelip = self._physical_switchs[physicalswitch]
            protocol = conn.protocol
            try:
                method, params = ovsdb.transact('hardware_vtep',
                                                ovsdb.select('Physical_Switch',
                                                             [["name", "==", physicalswitch]],
                                                             ["_uuid", "name", "ports"]),
                                                ovsdb.select('Physical_Port',
                                                             [["name", "==", physicalport]],
                                                             ["_uuid", "name", "vlan_bindings"]),
                                                ovsdb.select('Logical_Switch',
                                                             [["name", "==", logicalnetwork]],
                                                             ["_uuid", "name"])
                                                )
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
                if 'rows' not in result[0]:
                    raise JsonRPCErrorResultException('select from Physical_Switch failed: ' + repr(result[0]))
                if 'rows' not in result[1]:
                    raise JsonRPCErrorResultException('select from Physical_Switch failed: ' + repr(result[1]))
                if 'rows' not in result[2]:
                    raise JsonRPCErrorResultException('select from Physical_Switch failed: ' + repr(result[2]))
                if not result[0]['rows']:
                    raise ValueError('Physical switch %r is not found, connection = %r' % (physicalswitch, conn))
                pswitch = result[0]['rows'][0]
                port_uuids = set(p[1] for p in ovsdb.getlist(pswitch['ports']))
                ports = [p for p in result[1]['rows'] if p['_uuid'][1] in port_uuids]
                if not ports:
                    raise ValueError('Physical port %r is not found in physical switch %r' % (physicalport, physicalswitch))
                port = ports[0]
                ls_list = result[2]['rows']
                if not ls_list:
                    # Network not exists, might already be unbinded, ignore
                    break
                ls_uuid = ls_list[0]['_uuid'][1]
                curr_network = ovsdb.omap_getvalue(port['vlan_bindings'], vlanid)
                if curr_network is None or curr_network[1] != ls_uuid:
                    # Not binded
                    break
                operations = []
                # First we must check that the situations are not changed
                operations.extend([ovsdb.wait("Physical_Switch",
                                              [["name", "==", physicalswitch]],
                                              ["_uuid"],
                                              [{"_uuid": pswitch['_uuid']}],
                                              timeout = 0),
                                   ovsdb.wait("Physical_Port",
                                              [["name", "==", physicalport],
                                               ["vlan_bindings", "includes", ovsdb.omap((vlanid, ovsdb.uuid(ls_uuid)))]],
                                              ["_uuid"],
                                              [{"_uuid": port['_uuid']}],
                                              timeout = 0),
                                   ovsdb.wait("Logical_Switch",
                                                [["name", "==", logicalnetwork]],
                                                ["_uuid"],
                                                [{"_uuid": ovsdb.uuid(ls_uuid)}],
                                                timeout = 0)])
                operations.append(ovsdb.mutate('Physical_Port', [["_uuid", "==", port['_uuid']]],
                                               [["vlan_bindings","delete",
                                                    ovsdb.omap(
                                                        (vlanid, ovsdb.uuid(ls_uuid))
                                                    )]]))
                # We do not delete the logical switch; it will be deleted by the recycling process if not used
                method, params = ovsdb.transact('hardware_vtep', *operations)
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
                if any(r for r in result[:3] if 'error' in r):
                    # Wait failed, the OVSDB is modified, retry
                    continue
                _check_transact_result(result[3:], operations[3:])
                break
            except ConnectionResetException:
                for m in self.apiroutine.waitWithTimeout(1):
                    yield m
                continue
            except IOError:
                for m in self.apiroutine.waitWithTimeout(1):
                    yield m
                continue
        self.apiroutine.retvalue = None
    
    def unbindphysicalport(self, physicalswitch, physicalport):
        '''
        Remove all bindings for a physical port

        :param physicalswitch: physical switch name, should be the name in PhysicalSwitch table of OVSDB vtep database
        
        :param physicalport: physical port name, should be the name in OVSDB vtep database        
        '''
        for m in self._wait_for_sync():
            yield m
        if physicalswitch is None:
            raise ValueError('Physical switch cannot be None')
        # We may retry some times
        while True:
            if physicalswitch not in self._physical_switchs:
                for m in self.apiroutine.waitWithTimeout(5,
                                                         VtepPhysicalSwitchStateChanged.createMatcher(
                                                            VtepPhysicalSwitchStateChanged.UP,
                                                            physicalswitch)):
                    yield m
                if self.apiroutine.timeout or physicalswitch not in self._physical_switchs:
                    raise ValueError('Physical switch %r is not found' % (physicalswitch,))
            conn, tunnelip = self._physical_switchs[physicalswitch]
            protocol = conn.protocol
            try:
                method, params = ovsdb.transact('hardware_vtep',
                                                ovsdb.select('Physical_Switch',
                                                             [["name", "==", physicalswitch]],
                                                             ["_uuid", "name", "ports"]),
                                                ovsdb.select('Physical_Port',
                                                             [["name", "==", physicalport]],
                                                             ["_uuid", "name", "vlan_bindings"])
                                                )
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
                if 'rows' not in result[0]:
                    raise JsonRPCErrorResultException('select from Physical_Switch failed: ' + repr(result[0]))
                if 'rows' not in result[1]:
                    raise JsonRPCErrorResultException('select from Physical_Switch failed: ' + repr(result[1]))
                if not result[0]['rows']:
                    raise ValueError('Physical switch %r is not found, connection = %r' % (physicalswitch, conn))
                pswitch = result[0]['rows'][0]
                port_uuids = set(p[1] for p in ovsdb.getlist(pswitch['ports']))
                ports = [p for p in result[1]['rows'] if p['_uuid'][1] in port_uuids]
                if not ports:
                    raise ValueError('Physical port %r is not found in physical switch %r' % (physicalport, physicalswitch))
                port = ports[0]
                # First we must check that the situations are not changed
                operations = [ovsdb.wait("Physical_Switch",
                                              [["name", "==", physicalswitch]],
                                              ["_uuid"],
                                              [{"_uuid": pswitch['_uuid']}],
                                              timeout = 0),
                               ovsdb.wait("Physical_Port",
                                          [["name", "==", physicalport]],
                                          ["_uuid"],
                                          [{"_uuid": port['_uuid']}],
                                          timeout = 0),
                               ovsdb.update("Physical_Port",
                                            [["_uuid", "==", port["_uuid"]]],
                                            {"vlan_bindings": ovsdb.omap()})]
                # We do not delete the logical switch; it will be deleted by the recycling process if not used
                method, params = ovsdb.transact('hardware_vtep', *operations)
                for m in protocol.querywithreply(method, params, conn, self.apiroutine):
                    yield m
                result = self.apiroutine.jsonrpc_result
                if any(r for r in result[:2] if 'error' in r):
                    # Wait failed, the OVSDB is modified, retry
                    continue
                _check_transact_result(result[2:], operations[2:])
                break
            except ConnectionResetException:
                for m in self.apiroutine.waitWithTimeout(1):
                    yield m
                continue
            except IOError:
                for m in self.apiroutine.waitWithTimeout(1):
                    yield m
                continue
        self.apiroutine.retvalue = None

